@@ -25,7 +25,7 @@ in this Software without prior written authorization from the X Consortium.
  * Author:  Keith Packard, MIT X Consortium
  */
 
-/* $XFree86: xc/programs/Xserver/Xext/saver.c,v 3.11 2004/10/23 01:28:38 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/Xext/saver.c,v 3.13 2005/10/14 15:16:11 tsi Exp $ */
 /*
  * Copyright (c) 1994-2004 by The XFree86 Project, Inc.
  * All rights reserved.
@@ -75,8 +75,8 @@ in this Software without prior written authorization from the X Consortium.
 
 #define NEED_REPLIES
 #define NEED_EVENTS
-#include "X.h"
-#include "Xproto.h"
+#include <X11/X.h>
+#include <X11/Xproto.h>
 #include "misc.h"
 #include "os.h"
 #include "windowstr.h"
@@ -86,7 +86,7 @@ in this Software without prior written authorization from the X Consortium.
 #include "dixstruct.h"
 #include "resource.h"
 #include "opaque.h"
-#include "saverproto.h"
+#include <X11/extensions/saverproto.h>
 #include "gcstruct.h"
 #include "cursorstr.h"
 #include "colormapst.h"
@@ -104,6 +104,7 @@ in this Software without prior written authorization from the X Consortium.
 
 #include "modinit.h"
 
+#ifdef SCREENSAVER
 #if 0
 static unsigned char ScreenSaverReqCode = 0;
 #endif
@@ -295,14 +296,12 @@ ScreenSaverExtensionInit(INITARGS)
 
 /*ARGSUSED*/
 static void
-ScreenSaverResetProc (extEntry)
-ExtensionEntry	*extEntry;
+ScreenSaverResetProc(ExtensionEntry *extEntry)
 {
 }
 
 static void
-CheckScreenPrivate (pScreen)
-    ScreenPtr	pScreen;
+CheckScreenPrivate(ScreenPtr pScreen)
 {
     SetupScreen (pScreen);
 
@@ -318,8 +317,7 @@ CheckScreenPrivate (pScreen)
 }
 
 static ScreenSaverScreenPrivatePtr
-MakeScreenPrivate (pScreen)
-    ScreenPtr	pScreen;
+MakeScreenPrivate(ScreenPtr pScreen)
 {
     SetupScreen (pScreen);
 
@@ -338,9 +336,7 @@ MakeScreenPrivate (pScreen)
 }
 
 static unsigned long
-getEventMask (pScreen, client)
-    ScreenPtr	pScreen;
-    ClientPtr	client;
+getEventMask(ScreenPtr pScreen, ClientPtr client)
 {
     SetupScreen(pScreen);
     ScreenSaverEventPtr	pEv;
@@ -354,10 +350,7 @@ getEventMask (pScreen, client)
 }
 
 static Bool
-setEventMask (pScreen, client, mask)
-    ScreenPtr	pScreen;
-    ClientPtr	client;
-    unsigned long   mask;
+setEventMask(ScreenPtr pScreen, ClientPtr client, unsigned long mask)
 {
     SetupScreen(pScreen);
     ScreenSaverEventPtr	pEv, *pPrev;
@@ -404,8 +397,7 @@ setEventMask (pScreen, client, mask)
 }
 
 static void
-FreeAttrs (pAttr)
-    ScreenSaverAttrPtr	pAttr;
+FreeAttrs(ScreenSaverAttrPtr pAttr)
 {
     PixmapPtr	    pPixmap;
     CursorPtr	    pCursor;
@@ -419,8 +411,7 @@ FreeAttrs (pAttr)
 }
 
 static void
-FreeScreenAttr (pAttr)
-    ScreenSaverAttrPtr	pAttr;
+FreeScreenAttr(ScreenSaverAttrPtr pAttr)
 {
     FreeAttrs (pAttr);
     xfree (pAttr->values);
@@ -428,9 +419,7 @@ FreeScreenAttr (pAttr)
 }
 
 static int
-ScreenSaverFreeEvents (value, id)
-    pointer value;
-    XID id;
+ScreenSaverFreeEvents(pointer value, XID id)
 {
     ScreenSaverEventPtr	pOld = (ScreenSaverEventPtr)value;
     ScreenPtr pScreen = pOld->screen;
@@ -451,9 +440,7 @@ ScreenSaverFreeEvents (value, id)
 }
 
 static int
-ScreenSaverFreeAttr (value, id)
-    pointer value;
-    XID id;
+ScreenSaverFreeAttr(pointer value, XID id)
 {
     ScreenSaverAttrPtr	pOldAttr = (ScreenSaverAttrPtr)value;
     ScreenPtr	pScreen = pOldAttr->screen;
@@ -475,10 +462,7 @@ ScreenSaverFreeAttr (value, id)
 }
 
 static void
-SendScreenSaverNotify (pScreen, state, forced)
-    ScreenPtr			pScreen;
-    int	    state;
-    Bool    forced;
+SendScreenSaverNotify(ScreenPtr pScreen, int state, Bool forced)
 {
     ScreenSaverScreenPrivatePtr	pPriv;
     ScreenSaverEventPtr		pEv;
@@ -521,8 +505,8 @@ SendScreenSaverNotify (pScreen, state, forced)
 }
 
 static void
-SScreenSaverNotifyEvent (from, to)
-    xScreenSaverNotifyEvent *from, *to;
+SScreenSaverNotifyEvent(xScreenSaverNotifyEvent *from,
+			xScreenSaverNotifyEvent *to)
 {
     to->type = from->type;
     to->state = from->state;
@@ -535,8 +519,7 @@ SScreenSaverNotifyEvent (from, to)
 }
 
 static void
-UninstallSaverColormap (pScreen)
-    ScreenPtr	pScreen;
+UninstallSaverColormap(ScreenPtr pScreen)
 {
     SetupScreen(pScreen);
     ColormapPtr			pCmap;
@@ -552,8 +535,7 @@ UninstallSaverColormap (pScreen)
 }
 
 static Bool
-CreateSaverWindow (pScreen)
-    ScreenPtr	pScreen;
+CreateSaverWindow(ScreenPtr pScreen)
 {
     SetupScreen (pScreen);
     ScreenSaverStuffPtr		pSaver;
@@ -672,8 +654,7 @@ CreateSaverWindow (pScreen)
 }
 
 static Bool
-DestroySaverWindow (pScreen)
-    ScreenPtr	pScreen;
+DestroySaverWindow(ScreenPtr pScreen)
 {
     SetupScreen(pScreen);
     ScreenSaverStuffPtr		pSaver;
@@ -694,10 +675,7 @@ DestroySaverWindow (pScreen)
 }
 
 static Bool
-ScreenSaverHandle (pScreen, xstate, force)
-    ScreenPtr	pScreen;
-    int		xstate;
-    Bool	force;
+ScreenSaverHandle(ScreenPtr pScreen, int xstate, Bool force)
 {
     int				state = 0;
     Bool			ret = FALSE;
@@ -728,11 +706,10 @@ ScreenSaverHandle (pScreen, xstate, force)
 }
 
 static int
-ProcScreenSaverQueryVersion (client)
-    register ClientPtr	client;
+ProcScreenSaverQueryVersion(ClientPtr client)
 {
     xScreenSaverQueryVersionReply	rep;
-    register int		n;
+    int		n;
 
     REQUEST_SIZE_MATCH (xScreenSaverQueryVersionReq);
     rep.type = X_Reply;
@@ -749,12 +726,11 @@ ProcScreenSaverQueryVersion (client)
 }
 
 int
-ProcScreenSaverQueryInfo (client)
-    register ClientPtr	client;
+ProcScreenSaverQueryInfo(ClientPtr client)
 {
     REQUEST(xScreenSaverQueryInfoReq);
     xScreenSaverQueryInfoReply	rep;
-    register int		n;
+    int		n;
     ScreenSaverStuffPtr		pSaver;
     DrawablePtr			pDraw;
     CARD32			lastInput;
@@ -821,8 +797,7 @@ ProcScreenSaverQueryInfo (client)
 }
 
 static int
-ProcScreenSaverSelectInput (client)
-    register ClientPtr	client;
+ProcScreenSaverSelectInput(ClientPtr client)
 {
     REQUEST(xScreenSaverSelectInputReq);
     DrawablePtr			pDraw;
@@ -837,7 +812,7 @@ ProcScreenSaverSelectInput (client)
 }
 
 static int
-ScreenSaverSetAttributes (ClientPtr client)
+ScreenSaverSetAttributes(ClientPtr client)
 {
     REQUEST(xScreenSaverSetAttributesReq);
     DrawablePtr			pDraw;
@@ -1216,7 +1191,7 @@ bail:
 }
 
 static int
-ScreenSaverUnsetAttributes (ClientPtr client)
+ScreenSaverUnsetAttributes(ClientPtr client)
 {
     REQUEST(xScreenSaverSetAttributesReq);
     DrawablePtr			pDraw;
@@ -1237,7 +1212,7 @@ ScreenSaverUnsetAttributes (ClientPtr client)
 }
 
 static int
-ProcScreenSaverSetAttributes (ClientPtr client)
+ProcScreenSaverSetAttributes(ClientPtr client)
 {
 #ifdef PANORAMIX
     if(!noPanoramiXExtension) {
@@ -1316,7 +1291,7 @@ ProcScreenSaverSetAttributes (ClientPtr client)
 }
 
 static int
-ProcScreenSaverUnsetAttributes (ClientPtr client)
+ProcScreenSaverUnsetAttributes(ClientPtr client)
 {
 #ifdef PANORAMIX
     if(!noPanoramiXExtension) {
@@ -1351,8 +1326,7 @@ static DISPATCH_PROC((*NormalVector[])) = {
 #define NUM_REQUESTS	((sizeof NormalVector) / (sizeof NormalVector[0]))
 
 static int
-ProcScreenSaverDispatch (client)
-    ClientPtr	client;
+ProcScreenSaverDispatch(ClientPtr client)
 {
     REQUEST(xReq);
 
@@ -1362,8 +1336,7 @@ ProcScreenSaverDispatch (client)
 }
 
 static int
-SProcScreenSaverQueryVersion (client)
-    ClientPtr	client;
+SProcScreenSaverQueryVersion(ClientPtr client)
 {
     REQUEST(xScreenSaverQueryVersionReq);
     int	    n;
@@ -1374,8 +1347,7 @@ SProcScreenSaverQueryVersion (client)
 }
 
 static int
-SProcScreenSaverQueryInfo (client)
-    ClientPtr	client;
+SProcScreenSaverQueryInfo(ClientPtr client)
 {
     REQUEST(xScreenSaverQueryInfoReq);
     int	    n;
@@ -1387,8 +1359,7 @@ SProcScreenSaverQueryInfo (client)
 }
 
 static int
-SProcScreenSaverSelectInput (client)
-    ClientPtr	client;
+SProcScreenSaverSelectInput(ClientPtr client)
 {
     REQUEST(xScreenSaverSelectInputReq);
     int	    n;
@@ -1401,8 +1372,7 @@ SProcScreenSaverSelectInput (client)
 }
 
 static int
-SProcScreenSaverSetAttributes (client)
-    ClientPtr	client;
+SProcScreenSaverSetAttributes(ClientPtr client)
 {
     REQUEST(xScreenSaverSetAttributesReq);
     int	    n;
@@ -1422,8 +1392,7 @@ SProcScreenSaverSetAttributes (client)
 }
 
 static int
-SProcScreenSaverUnsetAttributes (client)
-    ClientPtr	client;
+SProcScreenSaverUnsetAttributes(ClientPtr client)
 {
     REQUEST(xScreenSaverUnsetAttributesReq);
     int	    n;
@@ -1443,8 +1412,7 @@ static DISPATCH_PROC((*SwappedVector[])) = {
 };
 
 static int
-SProcScreenSaverDispatch (client)
-    ClientPtr	client;
+SProcScreenSaverDispatch(ClientPtr client)
 {
     REQUEST(xReq);
 
@@ -1452,3 +1420,5 @@ SProcScreenSaverDispatch (client)
 	return (*SwappedVector[stuff->data])(client);
     return BadRequest;
 }
+
+#endif
