@@ -1,4 +1,4 @@
-/* $Xorg: Init.c,v 1.5 2001/03/07 17:31:33 pookie Exp $ */
+/* $XFree86: xc/programs/Xserver/Xprint/Init.c,v 1.19 2007/01/23 18:02:56 tsi Exp $ */
 /*
 (c) Copyright 1996 Hewlett-Packard Company
 (c) Copyright 1996 International Business Machines Corp.
@@ -50,7 +50,6 @@ copyright holders.
 **    *********************************************************
 ** 
 ********************************************************************/
-/* $XFree86: xc/programs/Xserver/Xprint/Init.c,v 1.15 2003/10/29 22:11:54 tsi Exp $ */
 
 #include <unistd.h>
 #include <stdlib.h>
@@ -65,9 +64,9 @@ copyright holders.
 #include <sys/sysmacros.h>
 #endif
 
-#include "X.h"
+#include <X11/X.h>
 #define NEED_EVENTS 1
-#include "Xproto.h"
+#include <X11/Xproto.h>
 #include <servermd.h>
 
 #include "screenint.h"
@@ -78,14 +77,14 @@ copyright holders.
 #include "inputstr.h"
 
 #include "gcstruct.h"
-#include "fonts/fontstruct.h"
+#include <X11/fonts/fontstruct.h>
 #include "errno.h"
 
 typedef char *XPointer;
 #define HAVE_XPointer 1
 
 #define Status int
-#include <Xresource.h>
+#include <X11/Xresource.h>
 
 #include "DiPrint.h"
 #include "attributes.h"
@@ -96,12 +95,12 @@ static void GenericScreenInit(
     int index,
     ScreenPtr pScreen,
     int argc,
-    char **argv);
+    const char **argv);
 static Bool InitPrintDrivers(
     int index,
     ScreenPtr pScreen,
-    int argc,
-    char **argv);
+    const int argc,
+    const char **argv);
 
 /*
  * The following two defines are used to build the name "X*printers", where
@@ -325,9 +324,9 @@ typedef struct _driverMapping {
     int screenNum;
 } DriverMapEntry, *DriverMapPtr;
 
-static char *configFileName = (char *)NULL;
+static const char *configFileName = (char *)NULL;
 static Bool freeDefaultFontPath = FALSE;
-static char *origFontPath = (char *)NULL;
+static const char *origFontPath = (char *)NULL;
 
 /*
  * XprintOptions checks argv[i] to see if it is our command line
@@ -337,7 +336,7 @@ static char *origFontPath = (char *)NULL;
 int
 XprintOptions(
     int argc,
-    char **argv,
+    const char **argv,
     int i)
 {
     if(strcmp(argv[i], "-XpFile") == 0)
@@ -1228,13 +1227,21 @@ void
 PrinterInitOutput(
      ScreenInfo *pScreenInfo,
      int argc,
-     char **argv)
+     const char **argv)
 {
     PrinterDbPtr pDb, pDbEntry;
-    int driverCount = 0, i;
+    int driverCount, i;
     char **driverNames;
     char *configDir;
 
+    static char beenHere = FALSE;
+
+    if (beenHere)
+	return;
+    beenHere = TRUE;
+
+    driverCount = 0;
+
     /* 
      * this little test is just a warning at startup to make sure
      * that the config directory exists.
@@ -1392,8 +1399,8 @@ static Bool
 InitPrintDrivers(
     int index,
     ScreenPtr pScreen,
-    int argc,
-    char **argv)
+    const int argc,
+    const char **argv)
 {
     PrinterDbPtr pDb, pDb2;
 
@@ -1449,7 +1456,7 @@ GenericScreenInit(
      int index,
      ScreenPtr pScreen,
      int argc,
-     char **argv)
+     const char **argv)
 {
     float fWidth, fHeight, maxWidth, maxHeight;
     unsigned short width, height;
