@@ -2,7 +2,7 @@
  * cfb copy area
  */
 
-/* $XFree86: xc/programs/Xserver/cfb/cfbbitblt.c,v 1.20 2003/11/10 18:21:44 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/cfb/cfbbitblt.c,v 1.24 2006/12/25 15:35:16 tsi Exp $ */
 
 /*
 
@@ -31,11 +31,10 @@ in this Software without prior written authorization from The Open Group.
 Author: Keith Packard
 
 */
-/* $Xorg: cfbbitblt.c,v 1.4 2001/02/09 02:04:37 xorgcvs Exp $ */
 
-#include	"X.h"
-#include	"Xmd.h"
-#include	"Xproto.h"
+#include	<X11/X.h>
+#include	<X11/Xmd.h>
+#include	<X11/Xproto.h>
 #include	"gcstruct.h"
 #include	"windowstr.h"
 #include	"scrnintstr.h"
@@ -53,7 +52,9 @@ Author: Keith Packard
 #define cfbCopyPlane1toN cfbCopyPlane1to8
 #define cfbCopyPlaneNto1 cfbCopyPlane8to1
 #else
+#if IMAGE_BYTE_ORDER == LSBFirst
 static unsigned int FgPixel, BgPixel;
+#endif
 # if PSZ == 16
 #define cfbCopyPlane1toN cfbCopyPlane1to16
 #define cfbCopyPlaneNto1 cfbCopyPlane16to1
@@ -70,21 +71,9 @@ static unsigned int FgPixel, BgPixel;
 
 /* cfbBitBltcfb == cfbCopyPlaneExpand */
 RegionPtr
-cfbBitBlt (
-    register DrawablePtr pSrcDrawable,
-    register DrawablePtr pDstDrawable,
-    GC *pGC,
-    int srcx, int srcy,
-    int width, int height,
-    int dstx, int dsty,
-    void (*doBitBlt)(
-        DrawablePtr /*pSrc*/,
-        DrawablePtr /*pDst*/,
-        int /*alu*/,
-        RegionPtr /*prgnDst*/,
-        DDXPointPtr /*pptSrc*/,
-        unsigned long /*planemask*/),
-    unsigned long bitPlane)
+cfbBitBlt(DrawablePtr pSrcDrawable, DrawablePtr pDstDrawable, GC *pGC,
+	  int srcx, int srcy, int width, int height, int dstx, int dsty,
+	  cfbDoBitBltProcPtr doBitBlt, unsigned long bitPlane)
 {
     RegionPtr prgnSrcClip = NULL; /* may be a new region, or just a copy */
     Bool freeSrcClip = FALSE;
@@ -92,11 +81,11 @@ cfbBitBlt (
     RegionPtr prgnExposed;
     RegionRec rgnDst;
     DDXPointPtr pptSrc;
-    register DDXPointPtr ppt;
-    register BoxPtr pbox;
+    DDXPointPtr ppt;
+    BoxPtr pbox;
     int i;
-    register int dx;
-    register int dy;
+    int dx;
+    int dy;
     xRectangle origSource;
     DDXPointRec origDest;
     int numRects;
@@ -332,22 +321,10 @@ cfbBitBlt (
 
 
 RegionPtr
-cfbCopyPlaneReduce (
-    register DrawablePtr pSrcDrawable,
-    register DrawablePtr pDstDrawable,
-    GC *pGC,
-    int srcx, int srcy,
-    int width, int height,
-    int dstx, int dsty,
-    void (*doCopyPlane)(
-        DrawablePtr /*pSrc*/,
-        DrawablePtr /*pDst*/,
-        int /*alu*/,
-        RegionPtr /*prgnDst*/,
-        DDXPointPtr /*pptSrc*/,
-        unsigned long /*planemask*/,
-        unsigned long /*bitPlane*/),
-    unsigned long bitPlane)
+cfbCopyPlaneReduce(DrawablePtr pSrcDrawable, DrawablePtr pDstDrawable, GC *pGC,
+		   int srcx, int srcy, int width, int height,
+		   int dstx, int dsty, cfbDoCopyPlaneProcPtr doCopyPlane,
+		   unsigned long bitPlane)
 {
     RegionPtr prgnSrcClip = NULL; /* may be a new region, or just a copy */
     Bool freeSrcClip = FALSE;
@@ -355,11 +332,11 @@ cfbCopyPlaneReduce (
     RegionPtr prgnExposed;
     RegionRec rgnDst;
     DDXPointPtr pptSrc;
-    register DDXPointPtr ppt;
-    register BoxPtr pbox;
+    DDXPointPtr ppt;
+    BoxPtr pbox;
     int i;
-    register int dx;
-    register int dy;
+    int dx;
+    int dy;
     xRectangle origSource;
     DDXPointRec origDest;
     int numRects;
@@ -595,21 +572,10 @@ cfbCopyPlaneReduce (
 
 
 void
-cfbDoBitblt (pSrc, pDst, alu, prgnDst, pptSrc, planemask)
-    DrawablePtr	    pSrc, pDst;
-    int		    alu;
-    RegionPtr	    prgnDst;
-    DDXPointPtr	    pptSrc;
-    unsigned long   planemask;
+cfbDoBitblt(DrawablePtr pSrc, DrawablePtr  pDst, int alu, RegionPtr prgnDst,
+	    DDXPointPtr pptSrc, unsigned long planemask)
 {
-    void (*doBitBlt)(
-        DrawablePtr /*pSrc*/,
-        DrawablePtr /*pDst*/,
-        int /*alu*/,
-        RegionPtr /*prgnDst*/,
-        DDXPointPtr /*pptSrc*/,
-        unsigned long /*planemask*/)
-        = cfbDoBitbltGeneral;
+    cfbDoBitBltProcPtr doBitBlt = cfbDoBitbltGeneral;
 
     if ((planemask & PMSK) == PMSK) {
 	switch (alu) {
@@ -628,23 +594,12 @@ cfbDoBitblt (pSrc, pDst, alu, prgnDst, pptSrc, planemask)
 }
 
 RegionPtr
-cfbCopyArea(pSrcDrawable, pDstDrawable,
-            pGC, srcx, srcy, width, height, dstx, dsty)
-    register DrawablePtr pSrcDrawable;
-    register DrawablePtr pDstDrawable;
-    GC *pGC;
-    int srcx, srcy;
-    int width, height;
-    int dstx, dsty;
+cfbCopyArea(DrawablePtr pSrcDrawable, DrawablePtr pDstDrawable,
+            GC *pGC, int srcx, int srcy, int width, int height,
+	    int dstx, int dsty)
 {
-    void (*doBitBlt) (
-        DrawablePtr /*pSrc*/,
-        DrawablePtr /*pDst*/,
-        int /*alu*/,
-        RegionPtr /*prgnDst*/,
-        DDXPointPtr /*pptSrc*/,
-        unsigned long /*planemask*/);
-    
+    cfbDoBitBltProcPtr doBitBlt;
+
     doBitBlt = cfbDoBitbltCopy;
     if (pGC->alu != GXcopy || (pGC->planemask & PMSK) != PMSK)
     {
@@ -666,18 +621,24 @@ cfbCopyArea(pSrcDrawable, pDstDrawable,
 }
 
 #if PSZ == 8
+/*
+    DrawablePtr pSrcDrawable;	must be a bitmap
+    DrawablePtr pDstDrawable;	must be depth 8 drawable
+    int	rop;			not used; caller must call
+				cfb8CheckOpaqueStipple
+			 	beforehand to get cfb8StippleRRop set correctly
+    RegionPtr prgnDst;		region in destination to draw to;
+				screen relative coords. if dest is a window;
+				drawable relative if dest is a pixmap
+    DDXPointPtr pptSrc;		drawable relative src coords to copy from;
+				must be one point for each box in prgnDst
+    unsigned long planemask;	to apply to destination writes
+*/
+
 void
-cfbCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask)
-    DrawablePtr pSrcDrawable;	/* must be a bitmap */
-    DrawablePtr pDstDrawable;	/* must be depth 8 drawable */
-    int	rop;		/* not used; caller must call cfb8CheckOpaqueStipple
-			 * beforehand to get cfb8StippleRRop set correctly */
-    RegionPtr prgnDst;		/* region in destination to draw to;
-				 * screen relative coords. if dest is a window;
-				 * drawable relative if dest is a pixmap */
-    DDXPointPtr pptSrc;		/* drawable relative src coords to copy from;
-				 * must be one point for each box in prgnDst */
-    unsigned long planemask;	/* to apply to destination writes */
+cfbCopyPlane1to8(DrawablePtr pSrcDrawable, DrawablePtr pDstDrawable,
+		 int rop, RegionPtr prgnDst, DDXPointPtr pptSrc,
+		 unsigned long planemask)
 {
     int	srcx, srcy;	/* upper left corner of box being copied in source */
     int dstx, dsty;	/* upper left corner of box being copied in dest */
@@ -689,14 +650,14 @@ cfbCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask)
     int widthDst;    /* # of groups of 4 pixels (8 bits/pixel) in dst */
     CfbBits *psrcLine, *pdstLine; /* steps a row at a time thru src/dst; 
 					 * may point into middle of row */
-    register CfbBits *psrc, *pdst; /* steps within the row */
-    register CfbBits bits, tmp;	 /* bits from source */
-    register int leftShift;
-    register int rightShift;
+    CfbBits *psrc, *pdst; /* steps within the row */
+    CfbBits bits, tmp;	 /* bits from source */
+    int leftShift;
+    int rightShift;
     CfbBits startmask;		/* left edge pixel mask */
     CfbBits endmask;		/* right edge pixel mask */
-    register int nlMiddle;   /* number of words in middle of the row to draw */
-    register int nl;
+    int nlMiddle;   /* number of words in middle of the row to draw */
+    int nl;
     int firstoff = 0;
     int secondoff = 0;
     CfbBits src;
@@ -829,13 +790,6 @@ cfbCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask)
 # define StoreRopBits(o,c)  StoreRopBitsPlain(o,c)
 # define FirstStep(c)	Step(c)
 #else /* BITMAP_BIT_ORDER == LSBFirst */
-#if PGSZ == 64
-# define StoreBits(o,c)	StorePixels(pdst,o, (cfb8Pixels[c & 0xff]))
-# define StoreRopBits(o,c)  StoreRopPixels(pdst,o, \
-    (cfb8StippleAnd[c & 0xff]), \
-    (cfb8StippleXor[c & 0xff]))
-# define FirstStep(c)	c = BitLeft (c, 8);
-#else
 /* 0x3c is 0xf << 2 (4 bits, long word) */
 # define StoreBits(o,c)	StorePixels(pdst,o,*((CfbBits *)\
 			    (((char *) cfb8Pixels) + (c & 0x3c))))
@@ -843,7 +797,6 @@ cfbCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask)
     *((CfbBits *) (((char *) cfb8StippleAnd) + (c & 0x3c))), \
     *((CfbBits *) (((char *) cfb8StippleXor) + (c & 0x3c))))
 # define FirstStep(c)	c = BitLeft (c, 2);
-#endif /* PGSZ */
 #endif /* BITMAP_BIT_ORDER */
 
 		    StoreBits0(tmp);	FirstStep(tmp);
@@ -998,6 +951,8 @@ cfbCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask)
 
 /******************************************************************/
 
+#if IMAGE_BYTE_ORDER == LSBFirst
+
 static void
 #if PSZ == 16
 cfbCopyPlane1to16
@@ -1022,27 +977,27 @@ cfbCopyPlane1to32
     CfbBits *psrcBase, *pdstBase;
     int	widthSrc, widthDst;
     unsigned int *psrcLine;
-    register unsigned int *psrc;
+    unsigned int *psrc;
 #if PSZ == 16
     unsigned short *pdstLine;
-    register unsigned short *pdst;
+    unsigned short *pdst;
 #endif
 #if PSZ == 32
     unsigned int *pdstLine;
-    register unsigned int *pdst;
+    unsigned int *pdst;
 #endif
 #if PSZ == 24
     unsigned char *pdstLine;
-    register unsigned char *pdst;
+    unsigned char *pdst;
 #endif
-    register unsigned int  bits, tmp;
-    register unsigned int  fgpixel, bgpixel;
-    register unsigned int  src;
+    unsigned int  bits, tmp;
+    unsigned int  fgpixel, bgpixel;
+    unsigned int  src;
 #if PSZ == 24
-    register unsigned int  dst;
+    unsigned int  dst;
 #endif
-    register int  leftShift, rightShift;
-    register int  i, nl;
+    int  leftShift, rightShift;
+    int  i, nl;
     int nbox;
     BoxPtr pbox;
     int  result;
@@ -1104,7 +1059,7 @@ cfbCopyPlane1to32
 	pdstLine = (unsigned short *)pdstBase + dsty * widthDst + dstx;
 #endif
 #if PSZ == 24
-	pdstLine = (unsigned char *)pdstBase + dsty * widthDst + dstx * 3;
+	pdstLine = (unsigned char *)pdstBase + dsty * widthDst + dstx * PSZB;
 #endif
 #if PSZ == 32
 	pdstLine = (unsigned int *)pdstBase + dsty * widthDst + dstx;
@@ -1190,7 +1145,7 @@ cfbCopyPlane1to32
 		            *(pdst + 1) = bgpixel >> 8;
 		            *(pdst + 2) = bgpixel >> 16;
 		        }
-		        pdst += 3;
+		        pdst += PSZB;
 		        i++;
 		    }
 #endif
@@ -1242,7 +1197,7 @@ cfbCopyPlane1to32
 		            *(pdst + 1) = bgpixel >> 8;
 		            *(pdst + 2) = bgpixel >> 16;
 		        }
-		        pdst += 3;
+		        pdst += PSZB;
 #else
 		        *pdst = ((tmp >> (31 - i)) & 0x01) ? fgpixel : bgpixel;
 		        pdst++;
@@ -1282,7 +1237,7 @@ cfbCopyPlane1to32
 		  	        ((result & planemask) >> 8);
 			*(pdst+2) = ((dst & ~planemask) >> 16) |
 		  	        ((result & planemask) >> 16);
-			pdst += 3;
+			pdst += PSZB;
 #else
                         DoRop (result, rop, src, *pdst);
 
@@ -1322,7 +1277,7 @@ cfbCopyPlane1to32
 		  	        ((result & planemask) >> 8);
 			*(pdst+2) = ((dst & ~planemask) >> 16) |
 		  	        ((result & planemask) >> 16);
-			pdst += 3;
+			pdst += PSZB;
 #else
                         DoRop (result, rop, src, *pdst);
 
@@ -1337,19 +1292,16 @@ cfbCopyPlane1to32
     }
 }
 
+#endif  /* IMAGE_BYTE_ORDER == LSBFirst */
+
 #endif  /* PSZ == 8 */
 
 /* shared among all different cfb depths through linker magic */
 
-RegionPtr cfbCopyPlane(pSrcDrawable, pDstDrawable,
-	    pGC, srcx, srcy, width, height, dstx, dsty, bitPlane)
-    DrawablePtr 	pSrcDrawable;
-    DrawablePtr		pDstDrawable;
-    GCPtr		pGC;
-    int 		srcx, srcy;
-    int 		width, height;
-    int 		dstx, dsty;
-    unsigned long	bitPlane;
+RegionPtr
+cfbCopyPlane(DrawablePtr pSrcDrawable, DrawablePtr pDstDrawable, GCPtr pGC,
+	     int srcx, int srcy, int width, int height, int dstx, int dsty,
+	     unsigned long bitPlane)
 {
     RegionPtr	ret;
 
