@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/cfb/cfbmskbits.h,v 3.16 2005/10/14 15:16:19 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/cfb/cfbmskbits.h,v 3.21 2006/12/25 16:40:45 tsi Exp $ */
 /************************************************************
 Copyright 1987 by Sun Microsystems, Inc. Mountain View, CA.
 
@@ -33,11 +33,6 @@ THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #include	<X11/X.h>
 #include	<X11/Xmd.h>
 #include	"servermd.h"
-#if defined(XFREE86) || ( defined(__OpenBSD__) && defined(__alpha__) ) \
-	|| (defined(__bsdi__))
-#include	"xf86_ansic.h"
-#include	"compiler.h"
-#endif
 
 /*
  * ==========================================================================
@@ -58,6 +53,7 @@ THE USE OR PERFORMANCE OF THIS SOFTWARE.
  * name	    explanation
  * ----	    -----------
  * PSZ	    pixel size (in bits)
+ * PSZB     pixel size (in bytes)
  * PGSZ     pixel group size (in bits)
  * PGSZB    pixel group size (in bytes)
  * PGSZBMSK mask with lowest PGSZB bits set to 1
@@ -127,11 +123,16 @@ THE USE OR PERFORMANCE OF THIS SOFTWARE.
 #define PixelGroup CARD32
 #define PGSZB 4
 #endif /* PixelGroup */
+
+#if !defined(PGSZB) || (PGSZB != 4)
+# error "PGSZB is unspecified or no longer supported."
+#endif
     
 #ifndef CfbBits
 #define CfbBits	CARD32
 #endif
 
+#define PSZB	(PSZ >> 3)
 #define PGSZ	(PGSZB << 3)
 #define PPW	(PGSZ/PSZ)
 #define PLST	(PPW-1)
@@ -409,30 +410,31 @@ getleftbits(psrc, w, dst)
 
 # if (PSZ == 24 && PPW == 1)
 #define maskbits(x, w, startmask, endmask, nlw) {\
-    startmask = cfbstarttab[(x)&3]; \
-    endmask = cfbendtab[((x)+(w)) & 3]; \
-    nlw = ((((x)+(w))*3)>>2) - (((x)*3 +3)>>2); \
+    startmask = cfbstarttab[(x) & (PGSZB - 1)]; \
+    endmask = cfbendtab[((x) + (w)) & (PGSZB - 1)]; \
+    nlw = ((((x) + (w)) * PSZB) / PGSZB) - ((((x) + 1) * PSZB) / PGSZB); \
 }
 
 #define mask32bits(x, w, startmask, endmask) \
-    startmask = cfbstarttab[(x)&3]; \
-    endmask = cfbendtab[((x)+(w)) & 3];
+    startmask = cfbstarttab[(x) & (PGSZB - 1)]; \
+    endmask = cfbendtab[((x) + (w)) & (PGSZB - 1)];
 
 #define maskpartialbits(x, w, mask) \
-    mask = cfbstartpartial[(x) & 3] & cfbendpartial[((x)+(w)) & 3];
+    mask = cfbstartpartial[(x) & (PGSZB - 1)] & \
+	   cfbendpartial[((x) + (w)) & (PGSZB - 1)];
 
 #define maskbits24(x, w, startmask, endmask, nlw) \
-    startmask = cfbstarttab24[(x) & 3]; \
-    endmask = cfbendtab24[((x)+(w)) & 3]; \
+    startmask = cfbstarttab24[(x) & (PGSZB - 1)]; \
+    endmask = cfbendtab24[((x) + (w)) & (PGSZB - 1)]; \
     if (startmask){ \
-	nlw = (((w) - (4 - ((x) & 3))) >> 2); \
+	nlw = (((w) - (PGSZB - ((x) & (PGSZB - 1)))) / PGSZB); \
     } else { \
-	nlw = (w) >> 2; \
+	nlw = (w) / PGSZB; \
     }
 
 #define getbits24(psrc, dst, index) {\
     int idx; \
-    switch(idx = ((index)&3)<<1){ \
+    switch(idx = ((index) & (PGSZB - 1)) << 1){ \
     	case 0: \
 		dst = (*(psrc) &cfbmask[idx]); \
 		break; \
@@ -448,7 +450,7 @@ getleftbits(psrc, w, dst)
 #define putbits24(src, w, pdst, planemask, index) {\
     PixelGroup dstpixel; \
     unsigned int idx; \
-    switch(idx = ((index)&3)<<1){ \
+    switch(idx = ((index) & (PGSZB - 1)) << 1){ \
     	case 0: \
 		dstpixel = (*(pdst) &cfbmask[idx]); \
 		break; \
@@ -574,8 +576,10 @@ else \
 #if defined(__GNUC__) && defined(mc68020)
 #undef getbits
 #define FASTGETBITS(psrc, x, w, dst) \
-    asm ("bfextu %3{%1:%2},%0" \
-	 : "=d" (dst) : "di" (x), "di" (w), "o" (*(char *)(psrc)))
+    __asm__ __volatile__ ( \
+	"bfextu %3{%1:%2},%0" \
+	: "=d" (dst) \
+	: "di" (x), "di" (w), "o" (*(char *)(psrc)))
 
 #define getbits(psrc,x,w,dst) \
 { \
@@ -584,9 +588,10 @@ else \
 }
 
 #define FASTPUTBITS(src, x, w, pdst) \
-    asm ("bfins %3,%0{%1:%2}" \
-	 : "=o" (*(char *)(pdst)) \
-	 : "di" (x), "di" (w), "d" (src), "0" (*(char *) (pdst)))
+    __asm__ __volatile__ ( \
+	"bfins %3,%0{%1:%2}" \
+	: "+o" (*(char *)(pdst)) \
+	: "di" (x), "di" (w), "d" (src))
 
 #undef putbits
 #define putbits(src, x, w, pdst, planemask) \
@@ -727,20 +732,20 @@ if ((x) + (w) <= PPW) {\
 }
 
 #if GETLEFTBITS_ALIGNMENT == 1
-#define getleftbits(psrc, w, dst)	dst = *((unsigned int *) psrc)
+#define getleftbits(psrc, w, dst) dst = *((unsigned int *) psrc)
 #define getleftbits24(psrc, w, dst, idx){	\
 	regiseter int index; \
-	switch(index = ((idx)&3)<<1){ \
+	switch(index = ((idx) & (PGSZB - 1)) << 1){ \
 	case 0: \
-	dst = (*((unsigned int *) psrc))&cfbmask[index]; \
+	dst = *((unsigned int *) psrc) & cfbmask[index]; \
 	break; \
 	case 2: \
 	case 4: \
-	dst = BitLeft(((*((unsigned int *) psrc))&cfbmask[index]), cfb24Shift[index]); \
-	dst |= BitRight(((*((unsigned int *) psrc)+1)&cfbmask[index]), cfb4Shift[index]); \
+	dst = BitLeft((*((unsigned int *) psrc) & cfbmask[index]), cfb24Shift[index]); \
+	dst |= BitRight(((*((unsigned int *) psrc) + 1) & cfbmask[index]), cfb4Shift[index]); \
 	break; \
 	case 6: \
-	dst = BitLeft((*((unsigned int *) psrc)),cfb24Shift[index]); \
+	dst = BitLeft(*((unsigned int *) psrc), cfb24Shift[index]); \
 	break; \
 	}; \
 }
@@ -749,16 +754,17 @@ if ((x) + (w) <= PPW) {\
 #define getglyphbits(psrc, x, w, dst) \
 { \
     dst = BitLeft((unsigned) *(psrc), (x)); \
-    if ( ((x) + (w)) > 32) \
+    if (((x) + (w)) > 32) \
 	dst |= (BitRight((unsigned) *((psrc)+1), 32-(x))); \
 }
 #if GETLEFTBITS_ALIGNMENT == 2
 #define getleftbits(psrc, w, dst) \
     { \
-	if ( ((int)(psrc)) & 0x01 ) \
-		getglyphbits( ((unsigned int *)(((char *)(psrc))-1)), 8, (w), (dst) ); \
+	if (((int)(psrc)) & 0x01 ) \
+	    getglyphbits(((unsigned int *)(((char *)(psrc)) - 1)), 8, (w), \
+			 (dst)); \
 	else \
-		dst = *((unsigned int *) psrc); \
+	    dst = *((unsigned int *) psrc); \
     }
 #endif /* GETLEFTBITS_ALIGNMENT == 2 */
 
@@ -766,11 +772,9 @@ if ((x) + (w) <= PPW) {\
 #define getleftbits(psrc, w, dst) \
     { \
 	int off, off_b; \
-	off_b = (off = ( ((int)(psrc)) & 0x03)) << 3; \
-	getglyphbits( \
-		(unsigned int *)( ((char *)(psrc)) - off), \
-		(off_b), (w), (dst) \
-	       ); \
+	off_b = (off = (((int)(psrc)) & (PGSZB - 1))) << 3; \
+	getglyphbits((unsigned int *)(((char *)(psrc)) - off), (off_b), (w), \
+		     (dst)); \
     }
 #endif /* GETLEFTBITS_ALIGNMENT == 4 */
 
@@ -823,15 +827,15 @@ if ((x) + (w) <= PPW) {\
 }
 #else /* BITMAP_BIT_ORDER == LSB */
 
-/* this must load 32 bits worth; for most machines, thats an int */
-#define CfbFetchUnaligned(x)	ldl_u(x)
-
 #define getstipplepixels( psrcstip, xt, w, ones, psrcpix, destpix ) \
 { \
-    PixelGroup q; \
-    q = CfbFetchUnaligned(psrcstip) >> (xt); \
-    if ( ((xt)+(w)) > (PPW*PSZ) ) \
-        q |= (CfbFetchUnaligned((psrcstip)+1)) << ((PPW*PSZ)-(xt)); \
+    PixelGroup q, tmp; \
+    memmove(&tmp, psrcstip, sizeof(PixelGroup)); \
+    q = tmp >> (xt); \
+    if ( ((xt)+(w)) > (PPW*PSZ) ) { \
+        memmove(&tmp, (psrcstip) + 1, sizeof(PixelGroup)); \
+        q |= tmp << ((PPW*PSZ)-(xt)); \
+    } \
     q = QuartetBitsTable[(w)] & ((ones) ? q : ~q); \
     *(destpix) = (*(psrcpix)) & QuartetPixelMaskTable[q]; \
 }
@@ -843,12 +847,12 @@ if ((x) + (w) <= PPW) {\
     CfbBits src; \
     unsigned int sidx; \
     unsigned int didx; \
-    sidx = ((srcindex) & 3)<<1; \
-    didx = ((dstindex) & 3)<<1; \
+    sidx = ((srcindex) & (PGSZB - 1)) << 1; \
+    didx = ((dstindex) & (PGSZB - 1)) << 1; \
     q = *(psrcstip) >> (xt); \
-/*    if((srcindex)!=0)*/ \
-/*    src = (((*(psrcpix)) << cfb24Shift[sidx]) & (cfbmask[sidx])) |*/ \
-/*	(((*((psrcpix)+1)) << cfb24Shift[sidx+1]) & (cfbmask[sidx+1])); */\
+/*    if((srcindex)!=0) */ \
+/*    src = (((*(psrcpix)) << cfb24Shift[sidx]) & (cfbmask[sidx])) | */ \
+/*	(((*((psrcpix)+1)) << cfb24Shift[sidx+1]) & (cfbmask[sidx+1])); */ \
 /*    else */\
 	src = (*(psrcpix))&0xFFFFFF; \
     if ( ((xt)+(w)) > PGSZ ) \
