@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/dix/main.c,v 3.50 2006/03/16 21:43:59 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/dix/main.c,v 3.55 2007/02/05 14:54:15 tsi Exp $ */
 /***********************************************************
 
 Copyright 1987, 1998  The Open Group
@@ -228,7 +228,7 @@ static int indexForScanlinePad[ 65 ] = {
 #endif
 
 int
-main(int argc, char *argv[], char *envp[])
+main(int argc, const char *argv[], char *envp[])
 {
     int		i, j, k, error;
     char	*xauthfile;
@@ -250,6 +250,9 @@ main(int argc, char *argv[], char *envp[])
     else
 	restart = 1;
 
+    /* Setup the allocator so that xalloc/xfree can be used early. */
+    OsInitAllocator();
+
     CheckUserParameters(argc, argv, envp);
 
     CheckUserAuthorization();
@@ -469,16 +472,12 @@ main(int argc, char *argv[], char *envp[])
 	if (dispatchException & DE_TERMINATE)
 	{
 	    CloseWellKnownConnections();
-	}
-
-	OsCleanup((dispatchException & DE_TERMINATE) != 0);
-
-	if (dispatchException & DE_TERMINATE)
-	{
+	    OsCleanup(TRUE);
 	    ddxGiveUp();
 	    break;
 	}
 
+	OsCleanup(FALSE);
 	xfree(ConnectionInfo);
 	ConnectionInfo = NULL;
 	OsPrepareRestart();
@@ -653,7 +652,7 @@ with its screen number, a pointer to its ScreenRec, argc, and argv.
 */
 
 int
-AddScreen(ScrnInitProcPtr pfnInit, int argc, char **argv)
+AddScreen(ScrnInitProcPtr pfnInit, const int argc, const char **argv)
 {
 
     int i;
