@@ -5,7 +5,7 @@
  *
  **************************************************************/
 /*
- * Copyright (c) 2001-2003 Torrey T. Lyons. All Rights Reserved.
+ * Copyright (c) 2001-2004 Torrey T. Lyons. All Rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a
  * copy of this software and associated documentation files (the "Software"),
@@ -29,7 +29,7 @@
  * holders shall not be used in advertising or otherwise to promote the sale,
  * use or other dealings in this Software without prior written authorization.
  */
-/* $XFree86: xc/programs/Xserver/hw/darwin/darwin.c,v 1.60 2006/02/19 15:51:19 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/darwin/darwin.c,v 1.64 2007/04/03 00:21:07 tsi Exp $ */
 
 #include <X11/X.h>
 #include <X11/Xproto.h>
@@ -89,13 +89,16 @@ unsigned int            darwinDesiredWidth = 0, darwinDesiredHeight = 0;
 int                     darwinDesiredDepth = -1;
 int                     darwinDesiredRefresh = -1;
 char                    *darwinKeymapFile = "USA.keymapping";
+int                     darwinSyncKeymap = FALSE;
+int                     darwinSwapAltMeta = FALSE;
 
 // modifier masks for faking mouse buttons
 int                     darwinFakeMouse2Mask = NX_COMMANDMASK;
 int                     darwinFakeMouse3Mask = NX_ALTERNATEMASK;
 
-static DeviceIntPtr     darwinPointer;
-static DeviceIntPtr     darwinKeyboard;
+// devices
+DeviceIntPtr            darwinPointer = NULL;
+DeviceIntPtr            darwinKeyboard = NULL;
 
 // Common pixmap formats
 static PixmapFormatRec formats[] = {
@@ -145,8 +148,8 @@ DarwinPrintBanner()
   ErrorF(" (%s)", XF86_CUSTOM_VERSION);
 #endif
   ErrorF(" / X Window System\n");
-  ErrorF("(protocol Version %d, revision %d, vendor release %d)\n",
-         X_PROTOCOL, X_PROTOCOL_REVISION, VENDOR_RELEASE );
+  ErrorF("(protocol Version %d, revision %d)\n",
+         X_PROTOCOL, X_PROTOCOL_REVISION);
   ErrorF("Release Date: %s\n", XF86_DATE);
   ErrorF("\tIf the server is older than 6-12 months, or if your hardware is\n"
          "\tnewer than the above date, look for a newer version before\n"
@@ -181,8 +184,8 @@ static Bool DarwinSaveScreen(ScreenPtr pScreen, int on)
 static Bool DarwinAddScreen(
     int         index,
     ScreenPtr   pScreen,
-    int         argc,
-    char        **argv )
+    const int   argc,
+    const char  **argv )
 {
     int         bitsPerRGB, i, dpi;
     static int  foundIndex = 0;
@@ -347,7 +350,7 @@ static int DarwinMouseProc(
     DeviceIntPtr    pPointer,
     int             what )
 {
-    char map[6];
+    unsigned char map[6];
 
     switch (what) {
 
@@ -529,7 +532,7 @@ static int DarwinParseModifierList(
  * InitInput
  *  Register the keyboard and mouse devices
  */
-void InitInput( int argc, char **argv )
+void InitInput( const int argc, const char **argv )
 {
     darwinPointer = AddInputDevice(DarwinMouseProc, TRUE);
     RegisterPointerDevice( darwinPointer );
@@ -607,7 +610,7 @@ DarwinAdjustScreenOrigins(ScreenInfo *pScreenInfo)
  *  After other screen setup has been done, a mode specific
  *  SetupScreen function can be called to finalize screen setup.
  */
-void InitOutput( ScreenInfo *pScreenInfo, int argc, char **argv )
+void InitOutput( ScreenInfo *pScreenInfo, const int argc, const char **argv )
 {
     int i;
     static unsigned long generation = 0;
@@ -689,7 +692,7 @@ void OsVendorInit(void)
  *  not device dependent, otherwise Count of number of elements of argv
  *  that are part of a device dependent commandline option.
  */
-int ddxProcessArgument( int argc, char *argv[], int i )
+int ddxProcessArgument( int argc, const char *argv[], int i )
 {
     int numDone;
 
@@ -734,11 +737,16 @@ int ddxProcessArgument( int argc, char *argv[], int i )
         return 2;
     }
 
+    if ( !strcmp( argv[i], "-swapAltMeta" ) ) {
+        darwinSwapAltMeta = 1;
+        return 1;
+    }
+
     if ( !strcmp( argv[i], "-keymap" ) ) {
         if ( i == argc-1 ) {
             FatalError( "-keymap must be followed by a filename\n" );
         }
-        darwinKeymapFile = argv[i+1];
+        darwinKeymapFile = (char *)argv[i+1];
         return 2;
     }
 
@@ -747,6 +755,16 @@ int ddxProcessArgument( int argc, char *argv[], int i )
         return 1;
     }
 
+    if ( !strcmp( argv[i], "+synckeymap" ) ) {
+        darwinSyncKeymap = TRUE;
+        return 1;
+    }
+
+    if ( !strcmp( argv[i], "-synckeymap" ) ) {
+        darwinSyncKeymap = FALSE;
+        return 1;
+    }
+
     if ( !strcmp( argv[i], "-size" ) ) {
         if ( i >= argc-2 ) {
             FatalError( "-size must be followed by two numbers\n" );
