@@ -33,10 +33,10 @@
  * holders shall not be used in advertising or otherwise to promote the sale,
  * use or other dealings in this Software without prior written authorization.
  */
-/* $XFree86: xc/programs/Xserver/hw/darwin/iokit/xfIOKit.c,v 1.4 2004/06/17 23:27:41 torrey Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/darwin/iokit/xfIOKit.c,v 1.7 2007/04/03 00:21:08 tsi Exp $ */
 
-#include "X.h"
-#include "Xproto.h"
+#include <X11/X.h>
+#include <X11/Xproto.h>
 #include "os.h"
 #include "servermd.h"
 #include "inputstr.h"
@@ -59,7 +59,7 @@
 #include <IOKit/IOKitLib.h>
 #include <IOKit/hidsystem/IOHIDShared.h>
 #include <IOKit/graphics/IOGraphicsLib.h>
-#include <drivers/event_status_driver.h>
+#include <IOKit/hidsystem/event_status_driver.h>
 
 // Define this to work around bugs in the display drivers for
 // older PowerBook G3's. If the X server starts without this
@@ -651,7 +651,7 @@ Bool DarwinModeSetupScreen(
  */
 void DarwinModeInitOutput(
     int argc,
-    char **argv)
+    const char **argv)
 {
     static unsigned long    generation = 0;
     kern_return_t           kr;
@@ -740,7 +740,7 @@ void DarwinModeInitOutput(
  */
 void DarwinModeInitInput(
     int argc,
-    char **argv)
+    const char **argv)
 {
     kern_return_t           kr;
     int                     fd[2];
