@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/hw/sun/sunGX.c,v 1.10 2005/10/14 15:16:26 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/sun/sunGX.c,v 1.12 2007/01/02 01:24:12 tsi Exp $ */
 /*
 Copyright 1991, 1998  The Open Group
 
@@ -282,7 +282,7 @@ sunGXDoBitblt(pSrc, pDst, alu, prgnDst, pptSrc, planemask)
     unsigned long   planemask;
 {
     register sunGXPtr	gx = sunGXGetScreenPrivate (pSrc->pScreen);
-    register long r;
+    register int r;
     register BoxPtr pboxTmp;
     register DDXPointPtr pptTmp;
     register int nbox;
@@ -424,7 +424,7 @@ sunGXCopyArea(pSrcDrawable, pDstDrawable,
             pGC, srcx, srcy, width, height, dstx, dsty, sunGXDoBitblt, 0);
 }
 
-static unsigned long	copyPlaneFG, copyPlaneBG;
+static unsigned long copyPlaneFG, copyPlaneBG;
 
 static void
 sunGXCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask, bitPlane)
@@ -441,8 +441,8 @@ sunGXCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask,
     int			dstLastx, dstRightx;
     int			xoffSrc, widthSrc, widthRest;
     int			widthLast;
-    unsigned long	*psrcBase, *psrc;
-    unsigned long	bits, tmp;
+    CfbBits		*psrcBase, *psrc;
+    CfbBits		bits, tmp;
     register int	leftShift, rightShift;
     register int	nl, nlMiddle;
     int			nbox;
@@ -459,7 +459,7 @@ sunGXCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask,
 
     nbox = REGION_NUM_RECTS(prgnDst);
     pbox = REGION_RECTS(prgnDst);
-    gx->incx = 32;
+    gx->incx = BITMAP_SCANLINE_UNIT;
     gx->incy = 0;
     while (nbox--)
     {
@@ -474,14 +474,14 @@ sunGXCopyPlane1to8 (pSrcDrawable, pDstDrawable, rop, prgnDst, pptSrc, planemask,
 	pptSrc++;
 	if (!width)
 	    continue;
-	psrc = psrcBase + srcy * widthSrc + (srcx >> 5);
+	psrc = psrcBase + srcy * widthSrc + (srcx >> LOG2_BITMAP_PAD);
 	dstLastx--;
-	dstRightx = dstx + 31;
-	nlMiddle = (width + 31) >> 5;
-	widthLast = width & 31;
-	xoffSrc = srcx & 0x1f;
+	dstRightx = dstx + BITMAP_SCANLINE_UNIT - 1;
+	nlMiddle = (width + BITMAP_SCANLINE_UNIT - 1) >> 5;
+	widthLast = width & (BITMAP_SCANLINE_UNIT - 1);
+	xoffSrc = srcx & ((1 << LOG2_BITMAP_PAD) - 1);
 	leftShift = xoffSrc;
-	rightShift = 32 - leftShift;
+	rightShift = BITMAP_SCANLINE_UNIT - leftShift;
 	widthRest = widthSrc - nlMiddle;
 	if (widthLast)
 	    nlMiddle--;
@@ -1528,7 +1528,7 @@ sunGXPolyGlyphBlt (pDrawable, pGC, x, y, nglyph, ppci, pglyphBase)
     int		    h;
     int		    w;
     CharInfoPtr	    pci;
-    unsigned long   *bits;
+    unsigned int    *bits;
     register int    r;
     RegionPtr	    clip;
     BoxPtr	    extents;
@@ -1585,7 +1585,7 @@ sunGXPolyGlyphBlt (pDrawable, pGC, x, y, nglyph, ppci, pglyphBase)
 	gx->x1 = (x + pci->metrics.rightSideBearing) - 1;
 	gx->y0 = y - pci->metrics.ascent;
 	h = pci->metrics.ascent + pci->metrics.descent;
-	bits = (unsigned long *) pci->bits;
+	bits = (unsigned int *) pci->bits;
 	while (h--) {
 	    gx->font = *bits++;
 	}
@@ -1609,12 +1609,12 @@ sunGXTEGlyphBlt (pDrawable, pGC, x, y, nglyph, ppci, pglyphBase)
     int		    h, hTmp;
     FontPtr	    pfont = pGC->font;
     register int    r;
-    unsigned long   *char1, *char2, *char3, *char4;
+    unsigned int    *char1, *char2, *char3, *char4;
     int		    widthGlyphs, widthGlyph;
     BoxRec	    bbox;
     BoxPtr	    extents;
     RegionPtr	    clip;
-    unsigned long   rop;
+    unsigned int    rop;
 
     widthGlyph = FONTMAXBOUNDS(pfont,characterWidth);
     h = FONTASCENT(pfont) + FONTDESCENT(pfont);
@@ -1673,10 +1673,10 @@ sunGXTEGlyphBlt (pDrawable, pGC, x, y, nglyph, ppci, pglyphBase)
     {
 	widthGlyphs = widthGlyph << 2;
 	LoopIt(4, widthGlyphs,
-	    char1 = (unsigned long *) (*ppci++)->bits;
-	    char2 = (unsigned long *) (*ppci++)->bits;
-	    char3 = (unsigned long *) (*ppci++)->bits;
-	    char4 = (unsigned long *) (*ppci++)->bits;,
+	    char1 = (unsigned int *) (*ppci++)->bits;
+	    char2 = (unsigned int *) (*ppci++)->bits;
+	    char3 = (unsigned int *) (*ppci++)->bits;
+	    char4 = (unsigned int *) (*ppci++)->bits;,
 	    (*char1++ | ((*char2++ | ((*char3++ | (*char4++
 		    >> widthGlyph))
 		    >> widthGlyph))
@@ -1686,17 +1686,17 @@ sunGXTEGlyphBlt (pDrawable, pGC, x, y, nglyph, ppci, pglyphBase)
     {
 	widthGlyphs = (widthGlyph << 1) + widthGlyph;
 	LoopIt(3, widthGlyphs,
-	    char1 = (unsigned long *) (*ppci++)->bits;
-	    char2 = (unsigned long *) (*ppci++)->bits;
-	    char3 = (unsigned long *) (*ppci++)->bits;,
+	    char1 = (unsigned int *) (*ppci++)->bits;
+	    char2 = (unsigned int *) (*ppci++)->bits;
+	    char3 = (unsigned int *) (*ppci++)->bits;,
 	    (*char1++ | ((*char2++ | (*char3++ >> widthGlyph)) >> widthGlyph)))
     }
     else if (widthGlyph <= 16)
     {
 	widthGlyphs = widthGlyph << 1;
 	LoopIt(2, widthGlyphs,
-	    char1 = (unsigned long *) (*ppci++)->bits;
-	    char2 = (unsigned long *) (*ppci++)->bits;,
+	    char1 = (unsigned int *) (*ppci++)->bits;
+	    char2 = (unsigned int *) (*ppci++)->bits;,
 	    (*char1++ | (*char2++ >> widthGlyph)))
     }
     while (nglyph--) {
@@ -1705,7 +1705,7 @@ sunGXTEGlyphBlt (pDrawable, pGC, x, y, nglyph, ppci, pglyphBase)
 	gx->x0 = x;
 	gx->x1 = (x += widthGlyph) - 1;
 	gx->y0 = y;
-	char1 = (unsigned long *) (*ppci++)->bits;
+	char1 = (unsigned int *) (*ppci++)->bits;
 	hTmp = h;
 	while (hTmp--)
 	    gx->font = *char1++;
@@ -1845,7 +1845,7 @@ sunGXCheckStipple (pPixmap, stipple)
     sunGXStipplePtr stipple;
 {
     unsigned short  *sbits;
-    unsigned long   *stippleBits;
+    unsigned int    *stippleBits;
     unsigned long   sbit, mask;
     int		    h, w;
     int		    y;
@@ -1858,7 +1858,7 @@ sunGXCheckStipple (pPixmap, stipple)
     if (w > 16 || (w & (w - 1)))
 	return FALSE;
     sbits = (unsigned short *) stipple->bits;
-    stippleBits = (unsigned long *) pPixmap->devPrivate.ptr;
+    stippleBits = (unsigned int *) pPixmap->devPrivate.ptr;
     mask = ((1 << w) - 1) << (16 - w);
     for (y = 0; y < h; y++) {
 	sbit = (*stippleBits++ >> 16) & mask;
@@ -2646,7 +2646,7 @@ sunGXChangeWindowAttributes (pWin, mask)
 		xfree (stipple);
 		sunGXSetWindowPrivate(pWin,0);
 	    }	    
- 	    if (((width = (pWin->background.pixmap->drawable.width * PSZ)) <= 32) &&
+ 	    if (((width = (pWin->background.pixmap->drawable.width * PSZ)) <= BITMAP_SCANLINE_UNIT) &&
 		       !(width & (width - 1)))
 	    {
 		cfbCopyRotatePixmap(pWin->background.pixmap,
@@ -2674,7 +2674,7 @@ sunGXChangeWindowAttributes (pWin, mask)
 
 	case CWBorderPixmap:
 	    /* don't bother with accelerator for border tiles (just lazy) */
-	    if (((width = (pWin->border.pixmap->drawable.width * PSZ)) <= 32) &&
+	    if (((width = (pWin->border.pixmap->drawable.width * PSZ)) <= BITMAP_SCANLINE_UNIT) &&
 		!(width & (width - 1)))
 	    {
 		for (pBgWin = pWin;
@@ -2848,7 +2848,7 @@ sunGXInit (
 {
     sunGXPtr	    gx;
     Uint	    mode;
-    register long   r;
+    register int   r;
 
     if (serverGeneration != sunGXGeneration)
     {
