@@ -1,4 +1,3 @@
-/* $Xorg: sunInit.c,v 1.4 2000/08/17 19:48:29 cpqbld Exp $ */
 /*
  * sunInit.c --
  *	Initialization functions for screen/keyboard/mouse, etc.
@@ -15,7 +14,7 @@
  *
  *
  */
-/* $XFree86: xc/programs/Xserver/hw/sun/sunInit.c,v 3.13 2003/11/17 22:20:36 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/sun/sunInit.c,v 3.20 2007/01/23 18:02:57 tsi Exp $ */
 
 /************************************************************
 Copyright 1987 by Sun Microsystems, Inc. Mountain View, CA.
@@ -66,7 +65,7 @@ extern Bool sunBW2Init(
     int /* screen */,
     ScreenPtr /* pScreen */,
     int /* argc */,
-    char** /* argv */
+    const char** /* argv */
 );
 #define BW2I sunBW2Init
 #endif /* } */
@@ -87,27 +86,27 @@ extern Bool sunCG3Init(
     int /* screen */,
     ScreenPtr /* pScreen */,
     int /* argc */,
-    char** /* argv */
+    const char** /* argv */
 );
 #define CG3I sunCG3Init
 #if defined(i386) || defined(__bsdi__) /* { */
 #define CG2I NULL
 #define CG4I NULL
 #else /* }{ */
-#ifdef INCLUDE_CG2_HEADER
+#if defined(INCLUDE_CG2_HEADER) || !defined(SVR4)
 extern Bool sunCG2Init(
     int /* screen */,
     ScreenPtr /* pScreen */,
     int /* argc */,
-    char** /* argv */
+    const char** /* argv */
 );
 #define CG2I sunCG2Init
-#endif /* INCLUDE_CG2_HEADER */
+#endif /* INCLUDE_CG2_HEADER || !SVR4 */
 extern Bool sunCG4Init(
     int /* screen */,
     ScreenPtr /* pScreen */,
     int /* argc */,
-    char** /* argv */
+    const char** /* argv */
 );
 #define CG4I sunCG4Init
 #endif /* } */
@@ -116,7 +115,7 @@ extern Bool sunCG6Init(
     int /* screen */,
     ScreenPtr /* pScreen */,
     int /* argc */,
-    char** /* argv */
+    const char** /* argv */
 );
 #define CG6I sunCG6Init
 #else /* }{ */
@@ -127,7 +126,7 @@ extern Bool sunTCXInit(
     int /* screen */,
     ScreenPtr /* pScreen */,
     int /* argc */,
-    char** /* argv */
+    const char** /* argv */
 );
 #define TCXI sunTCXInit
 #else /* }{ */
@@ -139,7 +138,7 @@ extern Bool sunCG8Init(
     int /* screen */,
     ScreenPtr /* pScreen */,
     int /* argc */,
-    char** /* argv */
+    const char** /* argv */
 );
 #define CG8I sunCG8Init
 #else /* }{ */
@@ -187,9 +186,11 @@ sunFbDataRec sunFbData[XFBTYPE_LASTPLUSONE] = {
   { NULL, "SUN1BW        (bw1)" },
   { NULL, "SUN1COLOR     (cg1)" },
   { BW2I, "SUN2BW        (bw2)" },	
-#ifdef INCLUDE_CG2_HEADER
+#if defined(INCLUDE_CG2_HEADER) || !defined(SVR4)
   { CG2I, "SUN2COLOR     (cg2)" },
-#endif /* INCLUDE_CG2_HEADER */
+#else
+  { NULL, "SUN2COLOR     (cg2)" },
+#endif /* INCLUDE_CG2_HEADER || !defined(SVR4) */
   { NULL, "SUN2GP        (gp1/gp2)" },
   { NULL, "SUN5COLOR     (cg5/386i accel)" },
   { CG3I, "SUN3COLOR     (cg3)" },
@@ -204,21 +205,11 @@ sunFbDataRec sunFbData[XFBTYPE_LASTPLUSONE] = {
   { NULL, "SUNFB_VIDEO" },
   { NULL, "SUNGIFB" },
   { NULL, "SUNPLAS" },
-#ifdef FBTYPE_SUNGP3
   { NULL, "SUNGP3        (cg12/gs)" },
-#endif
-#ifdef FBTYPE_SUNGT
   { NULL, "SUNGT         (gt)" },
-#endif
-#ifdef FBTYPE_SUNLEO
   { NULL, "SUNLEO        (zx)" },
-#endif
-#ifdef FBTYPE_MDICOLOR
   { NULL, "MDICOLOR      (cgfourteen)" },
-#endif
-#ifdef XFBTYPE_TCX
   { TCXI, "TCX           (tcx)" },
-#endif
 #endif /* } */
 };
 
@@ -278,7 +269,7 @@ static PixmapFormatRec	formats[] = {
 #if SUNMAXDEPTH > 1
     ,{ 8, 8, BITMAP_SCANLINE_PAD} /* 8-bit deep */
 #if SUNMAXDEPTH > 8
-    ,{ 12, 24, BITMAP_SCANLINE_PAD } /* 12-bit deep */
+    ,{ 12, 16, BITMAP_SCANLINE_PAD } /* 12-bit deep */
     ,{ 24, 32, BITMAP_SCANLINE_PAD } /* 24-bit deep */
 #endif
 #endif
@@ -420,7 +411,7 @@ void sunNonBlockConsoleOff(
 
 static char** GetDeviceList (argc, argv)
     int		argc;
-    char	**argv;
+    const char	**argv;
 {
     int		i;
     char	*envList = NULL;
@@ -429,7 +420,7 @@ static char** GetDeviceList (argc, argv)
 
     for (i = 1; i < argc; i++)
 	if (strcmp (argv[i], "-dev") == 0 && i+1 < argc) {
-	    cmdList = argv[i + 1];
+	    cmdList = (char *)argv[i + 1];
 	    break;
 	}
     if (!cmdList)
@@ -509,6 +500,12 @@ static void getKbdType()
     FatalError ("Unsupported keyboard type %d\n", sunKbdPriv.type);
 }
 
+void OsVendorPreInit(
+    void
+)
+{
+}
+
 void OsVendorInit(
     void
 )
@@ -573,8 +570,8 @@ void OsVendorFatalError(void)
 
 void InitOutput(pScreenInfo, argc, argv)
     ScreenInfo 	  *pScreenInfo;
-    int     	  argc;
-    char    	  **argv;
+    const int     argc;
+    const char    **argv;
 {
     int     	i, scr;
     int		nonBlockConsole = 0;
@@ -661,8 +658,8 @@ void InitOutput(pScreenInfo, argc, argv)
  *-----------------------------------------------------------------------
  */
 void InitInput(argc, argv)
-    int     	  argc;
-    char    	  **argv;
+    const int     argc;
+    const char    **argv;
 {
     pointer	p, k;
     extern Bool mieqInit();
@@ -936,28 +933,3 @@ sunCfbScreenInit(pScreen, pbits, xsize, ysize, dpix, dpiy, width, bpp)
 
 #endif  /* SUNMAXDEPTH == 32 */
 #endif  /* SUNMAXDEPTH */
-
-#ifdef DPMSExtension
-/**************************************************************
- * DPMSSet(), DPMSGet(), DPMSSupported()
- *
- * stubs
- *
- ***************************************************************/
-
-void DPMSSet (level)
-    int level;
-{
-}
-
-int DPMSGet (level)
-    int* level;
-{
-    return -1;
-}
-
-Bool DPMSSupported ()
-{
-    return FALSE;
-}
-#endif
