@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiident.c,v 1.13 2004/12/31 16:07:06 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiident.c,v 1.16 2007/01/01 16:08:16 tsi Exp $ */
 /*
- * Copyright 1997 through 2005 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
+ * Copyright 1997 through 2007 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -32,18 +32,12 @@
 const char *ATIChipsetNames[] =
 {
     "ati",
-
-#ifndef AVOID_CPIO
-
     "ativga",
     "ibmvga",
     "ibm8514",
     "vgawonder",
     "mach8",
     "mach32",
-
-#endif /* AVOID_CPIO */
-
     "mach64",
     "rage128",
     "radeon"
@@ -52,9 +46,6 @@ const char *ATIChipsetNames[] =
 static SymTabRec ATIPublicChipsetNames[] =
 {
     {ATI_CHIPSET_ATI, "ati"},
-
-#ifndef AVOID_CPIO
-
     {ATI_CHIPSET_ATIVGA, "ativga"},
 #ifdef __MAYBE_NOT__
     {ATI_CHIPSET_IBMVGA, "ibmvga"},
@@ -62,9 +53,6 @@ static SymTabRec ATIPublicChipsetNames[] =
 #ifdef __NOT_YET__
     {ATI_CHIPSET_IBM8514, "ibm8514"},
 #endif
-
-#endif /* AVOID_CPIO */
-
     {-1, NULL}
 };
 
@@ -104,17 +92,11 @@ ATIIdentProbe
 
     static SymTabRec SpecificNames[] =
     {
-
-#ifndef AVOID_CPIO
-
         {ATI_CHIPSET_VGAWONDER, "vgawonder"},
 #ifdef __NOT_YET__
         {ATI_CHIPSET_MACH8, "mach8"},
 #endif
         {ATI_CHIPSET_MACH32, "mach32"},
-
-#endif /* AVOID_CPIO */
-
         {ATI_CHIPSET_MACH64, "mach64"},
         {ATI_CHIPSET_RAGE128, "rage128"},
         {ATI_CHIPSET_RADEON, "radeon"},
