@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiprobe.c,v 1.73 2006/03/19 19:06:35 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiprobe.c,v 1.77 2007/02/13 18:30:09 tsi Exp $ */
 /*
- * Copyright 1997 through 2006 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
+ * Copyright 1997 through 2007 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -115,6 +115,8 @@ typedef struct
     CARD16 Base;
     CARD8  Size;
     CARD8  Flag;
+    CARD8  Flag2;
+    CARD8  Pad[7];
 } PortRec, *PortPtr;
 
 /* BIOS definitions */
@@ -140,16 +142,21 @@ ATIScanPCIBases
 (
     PortPtr      *PCIPorts,
     int          *nPCIPort,
-    const int     Domain,
+    const int    Domain,
     const CARD32 *pBase,
     const int    *pSize,
+          int    nBARs,
+          int    minMask,
     const CARD8  ProbeFlag
 )
 {
     CARD16 Base;
-    int    i, j;
+    int    i;
 
-    for (i = 6;  --i >= 0;  pBase++, pSize++)
+    if (minMask == TRUE)
+        minMask = -1;
+
+    for (;  minMask >>= 1, --nBARs >= 0;  pBase++, pSize++)
     {
         if (*pBase & PCI_MAP_IO)
         {
@@ -157,22 +164,26 @@ ATIScanPCIBases
                 continue;
 
             Base = *pBase & ~IO_BYTE_SELECT;
-            for (j = 0;  ;  j++)
+            for (i = 0;  ;  i++)
             {
-                if (j >= *nPCIPort)
+                if (i >= *nPCIPort)
                 {
                     (*nPCIPort)++;
                     *PCIPorts = (PortPtr)xnfrealloc(*PCIPorts,
                         *nPCIPort * SizeOf(PortRec));
-                    (*PCIPorts)[j].Domain = Domain;
-                    (*PCIPorts)[j].Base = Base;
-                    (*PCIPorts)[j].Size = (CARD8)*pSize;
-                    (*PCIPorts)[j].Flag = ProbeFlag;
+                    (*PCIPorts)[i].Domain = Domain;
+                    (*PCIPorts)[i].Base = Base;
+                    (*PCIPorts)[i].Size = (CARD8)*pSize;
+                    (*PCIPorts)[i].Flag = ProbeFlag;
+                    if (minMask & 1)
+                        (*PCIPorts)[i].Flag2 = ProbeFlag;
+                    else
+                        (*PCIPorts)[i].Flag2 = Allowed;
                     break;
                 }
 
-                if ((Domain == (*PCIPorts)[j].Domain) &&
-                    (Base == (*PCIPorts)[j].Base))
+                if ((Domain == (*PCIPorts)[i].Domain) &&
+                    (Base == (*PCIPorts)[i].Base))
                     break;
             }
 
@@ -183,7 +194,7 @@ ATIScanPCIBases
         if (!PCI_MAP_IS64BITMEM(*pBase))
             continue;
 
-        i--;
+        nBARs--;
         pBase++;
         pSize++;
     }
@@ -1132,7 +1143,7 @@ ATIProbe
     IOADDRESS           *pDomainIOBase = NULL;
     PortPtr             PCIPorts = NULL;
     CARD8               *ProbeFlags = NULL;
-    int                 ProbeSize;
+    int                 ProbeSize, nBARs;
     int                 nPCIPort = 0;
     int                 Domain, MaxDomain = 0;
     int                 i, j, k;
@@ -1147,7 +1158,7 @@ ATIProbe
     static const CARD16 Mach64SparseIOBases[] = {0x02ECU, 0x01CCU, 0x01C8U};
     CARD16              Mach64SparseIOBase;
     CARD8               fChipsets[ATI_CHIPSET_MAX];
-    CARD8               ProbeFlag;
+    CARD8               ProbeFlag, ProbeFlag2;
 
     unsigned long       BIOSBase;
     CARD8               BIOS[PrefixSize];
@@ -1257,10 +1268,15 @@ ATIProbe
                 if ((pVideo->vendor == PCI_VENDOR_ATI))
                     continue;
 
+                if (!(pciReadLong(pPCI->tag, PCI_CMD_STAT_REG) &
+                      PCI_CMD_IO_ENABLE))
+                    ProbeFlag = Allowed;
+                else
+                    ProbeFlag = 0;
+
                 ATIScanPCIBases(&PCIPorts, &nPCIPort,
                     Domain, &pPCI->pci_base0, pVideo->size,
-                    (pciReadLong(pPCI->tag, PCI_CMD_STAT_REG) &
-                     PCI_CMD_IO_ENABLE) ? 0 : Allowed);
+                    6, pPCI->minBasesize, ProbeFlag);
             }
         }
 
@@ -1273,16 +1289,37 @@ ATIProbe
                 if (Domain > MaxDomain)
                     MaxDomain = Domain;
 
-                if ((pPCI->pci_vendor == PCI_VENDOR_ATI) ||
-                    (pPCI->pci_base_class == PCI_CLASS_BRIDGE) ||
-                    (pPCI->pci_header_type &
-                      ~GetByte(PCI_HEADER_MULTIFUNCTION, 2)))
+                if (pPCI->pci_vendor == PCI_VENDOR_ATI)
                     continue;
 
+                switch (pPCI->pci_header_type &
+                        ~GetByte(PCI_HEADER_MULTIFUNCTION, 2))
+                {
+                    case 0:
+                        nBARs = 6;
+                        break;
+
+                    case 1:
+                        nBARs = 2;
+                        break;
+
+                    case 2:
+                        nBARs = 1;
+                        break;
+
+                    default:
+                        continue;
+                }
+
+                if (!(pciReadLong(pPCI->tag, PCI_CMD_STAT_REG) &
+                      PCI_CMD_IO_ENABLE))
+                    ProbeFlag = Allowed;
+                else
+                    ProbeFlag = Conflict;
+
                 ATIScanPCIBases(&PCIPorts, &nPCIPort,
                     Domain, &pPCI->pci_base0, pPCI->basesize,
-                    (pciReadLong(pPCI->tag, PCI_CMD_STAT_REG) &
-                     PCI_CMD_IO_ENABLE) ? Conflict : Allowed);
+                    nBARs, pPCI->minBasesize, ProbeFlag);
             }
         }
 
@@ -1299,6 +1336,7 @@ ATIProbe
 
             Domain = PCIPorts[i].Domain;
             ProbeFlag = PCIPorts[i].Flag;
+            ProbeFlag2 = PCIPorts[i].Flag2;
 
             /*
              * The following reduction of Count is based on the assumption that
@@ -1316,7 +1354,12 @@ ATIProbe
             Base = LongPort(Base) + (PCIPorts[i].Domain * DomainSize);
             Count = LongPort((Count | IO_BYTE_SELECT) + 1);
             while (Count--)
-                ProbeFlags[Base++] &= ProbeFlag;
+            {
+                if ((ProbeFlag != Allowed) || (ProbeFlags[Base] & Allowed))
+                    ProbeFlags[Base] = ProbeFlag;
+                Base++;
+                ProbeFlag = ProbeFlag2;
+            }
         }
 
         xfree(PCIPorts);
@@ -1868,7 +1911,8 @@ ATIProbe
                         continue;
 
                     case PCI_CLASS_DISPLAY:
-                        if (pVideo->subclass == PCI_SUBCLASS_DISPLAY_VGA)
+                        if ((pVideo->subclass == PCI_SUBCLASS_DISPLAY_VGA) &&
+                            (pVideo->interface == PCI_IF_DISPLAY_VGA))
                             break;
                         continue;
 
