@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiscreen.c,v 1.36 2006/01/05 18:55:30 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atiscreen.c,v 1.40 2007/01/23 18:02:59 tsi Exp $ */
 /*
- * Copyright 1999 through 2006 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
+ * Copyright 1999 through 2007 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -88,10 +88,10 @@ ATIRefreshArea
 Bool
 ATIScreenInit
 (
-    int       iScreen,
-    ScreenPtr pScreen,
-    int       argc,
-    char      **argv
+    int        iScreen,
+    ScreenPtr  pScreen,
+    const int  argc,
+    const char **argv
 )
 {
     ScrnInfoPtr  pScreenInfo = xf86Screens[iScreen];
