@@ -1,6 +1,6 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atixv.c,v 1.7 2004/12/31 16:07:07 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/ati/atixv.c,v 1.11 2007/01/01 16:08:18 tsi Exp $ */
 /*
- * Copyright 2001 through 2005 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
+ * Copyright 2001 through 2007 by Marc Aurele La France (TSI @ UQV), tsi@xfree86.org
  *
  * Permission to use, copy, modify, distribute, and sell this software and its
  * documentation for any purpose is hereby granted without fee, provided that
@@ -22,6 +22,7 @@
  */
 
 #include "atiadapter.h"
+#include "atichip.h"
 #include "atimach64xv.h"
 #include "atistruct.h"
 #include "atixv.h"
@@ -97,15 +98,10 @@ ATIXVPreInit
     ATIPtr      pATI
 )
 {
-
-#ifndef AVOID_CPIO
-
-    /* Currently a linear aperture is needed ... */
+    /* Currently a linear packed aperture is needed ... */
     if (!pATI->LinearBase)
         return;
 
-#endif /* AVOID_CPIO */
-
     (void)xf86XVRegisterGenericAdaptorDriver(ATIXVInitializeAdaptor);
 }
 
@@ -126,9 +122,14 @@ ATIInitializeXVideo
     int                 nAdaptor;
     Bool                result;
 
-    if (!(pScreenInfo->memPhysBase = pATI->LinearBase))
+    if ((pScreenInfo->depth < 8) ||
+        !(pScreenInfo->memPhysBase = pATI->LinearBase))
         return FALSE;
 
+    if ((ATIEndian.endian == ATI_BIG_ENDIAN) &&
+        (pATI->Chip >= ATI_CHIP_264VTB))
+        pScreenInfo->memPhysBase -= 0x00800000U;
+
     pScreenInfo->fbOffset = 0;
 
     nAdaptor = xf86XVListGenericAdaptors(pScreenInfo, &ppAdaptor);
