@@ -1,16 +1,12 @@
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nsc/nsc_driver.c,v 1.5 2003/04/23 21:51:41 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/nsc/nsc_driver.c,v 1.12 2006/12/10 15:58:30 tsi Exp $ */
 /*
- * $Workfile: nsc_driver.c $
- * $Revision: 21 $
- * $Author: Sarma $
- *
  * File Contents: This is the main module configures the interfacing 
  *                with the X server. The individual modules will be 
  *                loaded based upon the options selected from the 
  *                XF86Config. This file also has modules for finding 
  *                supported modes, turning on the modes based on options.
  *
- * Project:       Nsc Xfree Frame buffer device driver.
+ * Project:       Nsc XFree86 Frame buffer device driver.
  *
  */
 
@@ -143,9 +139,7 @@
  *
  * END_NSC_LIC_GPL */
 
-#define DEBUG(x)
 #define NSC_TRACE 0
-#define CFB 0
 #define HWVGA 1
 
 /* Includes that are used by all drivers */
@@ -167,20 +161,7 @@
 #define RC_MAX_DEPTH 24
 
 /* Frame buffer stuff */
-#if CFB
-/*
- * If using cfb, cfb.h is required.  Select the others for the bpp values
- * the driver supports.
- */
-#define PSZ 8				/* needed for cfb.h */
-#include "cfb.h"
-#undef PSZ
-#include "cfb16.h"
-#include "cfb24.h"
-#include "cfb32.h"
-#else
 #include "fb.h"
-#endif
 
 #include "shadowfb.h"
 
@@ -194,15 +175,13 @@
 #include "vbe.h"
 
 /* Check for some extensions */
-#ifdef XFreeXDGA
 #define _XF86_DGA_SERVER_
-#include "extensions/xf86dgastr.h"
-#endif /* XFreeXDGA */
+#include <X11/extensions/xf86dgastr.h>
 
 #include "globals.h"
 #include "opaque.h"
 #define DPMS_SERVER
-#include "extensions/dpms.h"
+#include <X11/extensions/dpms.h>
 
 #define EXTERN
 /* Our private include file (this also includes the durango headers) */
@@ -325,11 +304,10 @@ OptionInfoRec GeodeOptions[] = {
 const char *nscVgahwSymbols[] = {
    "vgaHWGetHWRec",
    "vgaHWUnlock",
-   "vgaHWInit",
    "vgaHWSave",
    "vgaHWRestore",
-   "vgaHWProtect",
    "vgaHWGetIOBase",
+   "vgaHWGetIndex",
    "vgaHWMapMem",
    "vgaHWLock",
    "vgaHWFreeHWRec",
@@ -352,21 +330,11 @@ const char *nscInt10Symbols[] = {
    NULL
 };
 
-#if CFB
-const char *nscCfbSymbols[] = {
-   "cfbScreenInit",
-   "cfb16ScreenInit",
-   "cfb24ScreenInit",
-   "cfb32ScreenInit",
-   NULL
-};
-#else
 const char *nscFbSymbols[] = {
    "fbScreenInit",
    "fbPictureInit",
    NULL
 };
-#endif
 
 const char *nscXaaSymbols[] = {
    "XAADestroyInfoRec",
@@ -431,7 +399,7 @@ XF86ModuleData nscModuleData = { &NscVersionRec, NscSetup, NULL };
  *-------------------------------------------------------------------------
 */
 static pointer
-NscSetup(pointer Module, pointer Options, int *ErrorMajor, int *ErrorMinor)
+NscSetup(ModuleDescPtr Module, pointer Options, int *ErrorMajor, int *ErrorMinor)
 {
    static Bool Initialised = FALSE;
 
@@ -441,15 +409,10 @@ NscSetup(pointer Module, pointer Options, int *ErrorMajor, int *ErrorMinor)
       /* Tell the loader about symbols from other modules that this
        * module might refer to.
        */
-      LoaderRefSymLists(nscVgahwSymbols, nscVbeSymbols,
-#if CFB
-			nscCfbSymbols,
-#else
-			nscFbSymbols,
-#endif
-			nscXaaSymbols,
-			nscInt10Symbols, nscRamdacSymbols, nscShadowSymbols,
-			NULL);
+      LoaderModRefSymLists(Module, nscVgahwSymbols, nscVbeSymbols,
+			   nscFbSymbols, nscXaaSymbols,
+			   nscInt10Symbols, nscRamdacSymbols, nscShadowSymbols,
+			   NULL);
       return (pointer) TRUE;
    }
    /*The return value must be non-NULL on success */
