@@ -24,7 +24,7 @@
  * USE OR OTHER DEALINGS IN THE SOFTWARE.
  *
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sunffb/ffb_gc.c,v 1.4 2003/06/23 17:35:48 eich Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/sunffb/ffb_gc.c,v 1.6 2006/03/01 21:01:56 dawes Exp $ */
 
 #include "ffb.h"
 #include "ffb_regs.h"
@@ -35,7 +35,7 @@
 
 #include "scrnintstr.h"
 #include "pixmapstr.h"
-#include "fontstruct.h"
+#include <X11/fonts/fontstruct.h>
 #include "dixfontstr.h"
 
 #define PSZ 8
@@ -443,7 +443,10 @@ CreatorNewFillArea(GCPtr pGC, cfbPrivGCPtr devPriv, CreatorPrivGCPtr gcPriv, int
 	pGC->ops->PushPixels = mfbPushPixels;
 }
 
-void
+extern GCOps cfbNonTEOps;
+extern GCOps cfb32NonTEOps;
+
+static void
 CreatorValidateGC (GCPtr pGC, unsigned long changes, DrawablePtr pDrawable)
 {
 	int	mask;		/* stateChanges */
@@ -462,9 +465,6 @@ CreatorValidateGC (GCPtr pGC, unsigned long changes, DrawablePtr pDrawable)
 	type = pFfb->vtSema ? -1 : pDrawable->type;
 	if (type != DRAWABLE_WINDOW) {
 		if (gcPriv->type == DRAWABLE_WINDOW) {
-			extern GCOps cfbNonTEOps;
-			extern GCOps cfb32NonTEOps;
-
 			miDestroyGCOps (pGC->ops);
 
 			if (pGC->depth == 8)
