@@ -20,7 +20,7 @@
  * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
  * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/suntcx/tcx_driver.c,v 1.12 2006/03/16 16:50:17 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/suntcx/tcx_driver.c,v 1.15 2007/03/15 02:11:37 tsi Exp $ */
 
 #include "xf86.h"
 #include "xf86_OSproc.h"
@@ -38,8 +38,8 @@ static const OptionInfoRec * TCXAvailableOptions(int chipid, int busid);
 static void	TCXIdentify(int flags);
 static Bool	TCXProbe(DriverPtr drv, int flags);
 static Bool	TCXPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool	TCXScreenInit(int Index, ScreenPtr pScreen, int argc,
-			      char **argv);
+static Bool	TCXScreenInit(int Index, ScreenPtr pScreen,
+			      const int argc, const char **argv);
 static Bool	TCXEnterVT(int scrnIndex, int flags);
 static void	TCXLeaveVT(int scrnIndex, int flags);
 static Bool	TCXCloseScreen(int scrnIndex, ScreenPtr pScreen);
@@ -466,7 +466,8 @@ TCXPreInit(ScrnInfoPtr pScrn, int flags)
 /* This gets called at the start of each server generation */
 
 static Bool
-TCXScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+TCXScreenInit(int scrnIndex, ScreenPtr pScreen,
+	      const int argc, const char **argv)
 {
     ScrnInfoPtr pScrn;
     TcxPtr pTcx;
@@ -741,11 +742,11 @@ TCXValidMode(int scrnIndex, DisplayModePtr mode, Bool verbose, int flags)
 /* Mandatory */
 static Bool
 TCXSaveScreen(ScreenPtr pScreen, int mode)
-    /* this function should blank the screen when unblank is FALSE and
-       unblank it when unblank is TRUE -- it doesn't actually seem to be
-       used for much though */
 {
-    return TRUE;
+    ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
+    TcxPtr pTcx = GET_TCX_FROM_SCRN(pScrn);
+
+    return xf86SbusSaveScreen(pTcx->psdp, mode);
 }
 
 /*
