@@ -28,7 +28,53 @@
  * Authors: Paulo Csar Pereira de Andrade <pcpa@conectiva.com.br>
  *          David Dawes <dawes@xfree86.org>
  *
- * $XFree86: xc/programs/Xserver/hw/xfree86/drivers/vesa/vesa.c,v 1.41 2003/11/07 22:50:57 dawes Exp $
+ * $XFree86: xc/programs/Xserver/hw/xfree86/drivers/vesa/vesa.c,v 1.57 2007/01/23 18:03:10 tsi Exp $
+ */
+/*
+ * Copyright (c) 2000-2005 by The XFree86 Project, Inc.
+ * All rights reserved.
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining
+ * a copy of this software and associated documentation files (the
+ * "Software"), to deal in the Software without restriction, including
+ * without limitation the rights to use, copy, modify, merge, publish,
+ * distribute, sublicense, and/or sell copies of the Software, and to
+ * permit persons to whom the Software is furnished to do so, subject
+ * to the following conditions:
+ *
+ *   1.  Redistributions of source code must retain the above copyright
+ *       notice, this list of conditions, and the following disclaimer.
+ *
+ *   2.  Redistributions in binary form must reproduce the above copyright
+ *       notice, this list of conditions and the following disclaimer
+ *       in the documentation and/or other materials provided with the
+ *       distribution, and in the same place and form as other copyright,
+ *       license and disclaimer information.
+ *
+ *   3.  The end-user documentation included with the redistribution,
+ *       if any, must include the following acknowledgment: "This product
+ *       includes software developed by The XFree86 Project, Inc
+ *       (http://www.xfree86.org/) and its contributors", in the same
+ *       place and form as other third-party acknowledgments.  Alternately,
+ *       this acknowledgment may appear in the software itself, in the
+ *       same form and location as other such third-party acknowledgments.
+ *
+ *   4.  Except as contained in this notice, the name of The XFree86
+ *       Project, Inc shall not be used in advertising or otherwise to
+ *       promote the sale, use or other dealings in this Software without
+ *       prior written authorization from The XFree86 Project, Inc.
+ *
+ * THIS SOFTWARE IS PROVIDED ``AS IS'' AND ANY EXPRESSED OR IMPLIED
+ * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
+ * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED.
+ * IN NO EVENT SHALL THE XFREE86 PROJECT, INC OR ITS CONTRIBUTORS BE
+ * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY,
+ * OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
+ * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR
+ * BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
+ * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE
+ * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
+ * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  */
 
 #include "vesa.h"
@@ -45,29 +91,29 @@
 
 /* DPMS */
 #define DPMS_SERVER
-#include "extensions/dpms.h"
+#include <X11/extensions/dpms.h>
 
 /* Mandatory functions */
 static const OptionInfoRec * VESAAvailableOptions(int chipid, int busid);
 static void VESAIdentify(int flags);
 static Bool VESAProbe(DriverPtr drv, int flags);
 static Bool VESAPreInit(ScrnInfoPtr pScrn, int flags);
-static Bool VESAScreenInit(int Index, ScreenPtr pScreen, int argc,
-			   char **argv);
+static Bool VESAScreenInit(int Index, ScreenPtr pScreen,
+			   const int argc, const char **argv);
 static Bool VESAEnterVT(int scrnIndex, int flags);
 static void VESALeaveVT(int scrnIndex, int flags);
 static Bool VESACloseScreen(int scrnIndex, ScreenPtr pScreen);
 static Bool VESASaveScreen(ScreenPtr pScreen, int mode);
 
 static Bool VESASwitchMode(int scrnIndex, DisplayModePtr pMode, int flags);
-static Bool VESASetMode(ScrnInfoPtr pScrn, DisplayModePtr pMode);
+static Bool VESASetMode(ScrnInfoPtr pScrn, DisplayModePtr pMode, Bool blank);
 static void VESAAdjustFrame(int scrnIndex, int x, int y, int flags);
 static void VESAFreeScreen(int scrnIndex, int flags);
 static void VESAFreeRec(ScrnInfoPtr pScrn);
 
 static void
 VESADisplayPowerManagementSet(ScrnInfoPtr pScrn, int mode,
-                int flags);
+		int flags);
 
 /* locally used functions */
 static int VESAFindIsaDevice(GDevPtr dev);
@@ -76,10 +122,10 @@ static void VESAUnmapVidMem(ScrnInfoPtr pScrn);
 static int VESABankSwitch(ScreenPtr pScreen, unsigned int iBank);
 static void VESALoadPalette(ScrnInfoPtr pScrn, int numColors, int *indices,
 			    LOCO *colors, VisualPtr pVisual);
-static void SaveFonts(ScrnInfoPtr pScrn);
-static void RestoreFonts(ScrnInfoPtr pScrn);
-static Bool 
-VESASaveRestore(ScrnInfoPtr pScrn, vbeSaveRestoreFunction function);
+static void VESASavePlanes(ScrnInfoPtr pScrn);
+static void VESARestorePlanes(ScrnInfoPtr pScrn);
+static Bool VESASaveRestore(ScrnInfoPtr pScrn,
+			    vbeSaveRestoreFunction function);
 
 static void *VESAWindowPlanar(ScreenPtr pScrn, CARD32 row, CARD32 offset,
 			      int mode, CARD32 *size, void *closure);
@@ -90,7 +136,7 @@ static void *VESAWindowWindowed(ScreenPtr pScrn, CARD32 row, CARD32 offset,
 
 static Bool VESADGAInit(ScrnInfoPtr pScrn, ScreenPtr pScreen);
 
-/* 
+/*
  * This contains the functions needed by the server after loading the
  * driver module.  It must be supplied, and gets added the driver list by
  * the Module Setup funtion in the dynamic case.  In the static case a
@@ -116,27 +162,29 @@ enum GenericTypes
 static SymTabRec VESAChipsets[] =
 {
     {CHIP_VESA_GENERIC, "vesa"},
-    {-1,		 NULL}
+    {-1,		NULL}
 };
 
 static PciChipsets VESAPCIchipsets[] = {
   { CHIP_VESA_GENERIC, PCI_CHIP_VGA, RES_SHARED_VGA },
-  { -1,		-1,	   RES_UNDEFINED },
+  { -1,		       -1,	     RES_UNDEFINED },
 };
 
 static IsaChipsets VESAISAchipsets[] = {
   {CHIP_VESA_GENERIC, RES_EXCLUSIVE_VGA},
-  {-1,		0 }
+  {-1,		      0 }
 };
 
 typedef enum {
     OPTION_SHADOW_FB,
-    OPTION_DFLT_REFRESH
+    OPTION_DFLT_REFRESH,
+    OPTION_VBE_BIG_ENDIAN
 } VESAOpts;
 
 static const OptionInfoRec VESAOptions[] = {
     { OPTION_SHADOW_FB,    "ShadowFB",		OPTV_BOOLEAN,	{0},	FALSE },
     { OPTION_DFLT_REFRESH, "DefaultRefresh",	OPTV_BOOLEAN,	{0},	FALSE },
+    { OPTION_VBE_BIG_ENDIAN, "VBEBigEndian",	OPTV_BOOLEAN,	{0},	FALSE },
     { -1,		   NULL,		OPTV_NONE,	{0},	FALSE }
 };
 
@@ -144,8 +192,8 @@ static const OptionInfoRec VESAOptions[] = {
  * List of symbols from other modules that this module references.  This
  * list is used to tell the loader that it is OK for symbols here to be
  * unresolved providing that it hasn't been told that they haven't been
- * told that they are essential via a call to xf86LoaderReqSymbols() or
- * xf86LoaderReqSymLists().  The purpose is this is to avoid warnings about
+ * told that they are essential via a call to xf86LoaderModReqSymbols() or
+ * xf86LoaderModReqSymLists().  The purpose is this is to avoid warnings about
  * unresolved symbols that are not required.
  */
 #ifdef XFree86LOADER
@@ -177,7 +225,9 @@ static const char *vbeSymbols[] = {
     "VBEBankSwitch",
     "VBEExtendedInit",
     "VBEFindSupportedDepths",
+    "VBEFreeVBEInfo",
     "VBEGetModeInfo",
+    "VBEGetModePool",
     "VBEGetVBEInfo",
     "VBEGetVBEMode",
     "VBEPrintModes",
@@ -195,14 +245,13 @@ static const char *vbeSymbols[] = {
     NULL
 };
 
-#ifdef XFree86LOADER
 static const char *ddcSymbols[] = {
     "xf86PrintEDID",
     "xf86SetDDCproperties",
     NULL
 };
 
-
+#ifdef XFree86LOADER
 /* Module loader interface */
 static MODULESETUPPROTO(vesaSetup);
 
@@ -227,7 +276,7 @@ static XF86ModuleVersionInfo vesaVersionRec =
 XF86ModuleData vesaModuleData = { &vesaVersionRec, vesaSetup, NULL };
 
 static pointer
-vesaSetup(pointer Module, pointer Options, int *ErrorMajor, int *ErrorMinor)
+vesaSetup(ModuleDescPtr Module, pointer Options, int *ErrorMajor, int *ErrorMinor)
 {
     static Bool Initialised = FALSE;
 
@@ -235,12 +284,13 @@ vesaSetup(pointer Module, pointer Options, int *ErrorMajor, int *ErrorMinor)
     {
 	Initialised = TRUE;
 	xf86AddDriver(&VESA, Module, 0);
-	LoaderRefSymLists(miscfbSymbols,
-			  fbSymbols,
-			  shadowSymbols,
-			  vbeSymbols,
-			  ddcSymbols,
-			  NULL);
+	LoaderModRefSymLists(Module,
+			     miscfbSymbols,
+			     fbSymbols,
+			     shadowSymbols,
+			     vbeSymbols,
+			     ddcSymbols,
+			     NULL);
 	return (pointer)TRUE;
     }
 
@@ -288,7 +338,7 @@ VESAProbe(DriverPtr drv, int flags)
     /* PCI BUS */
     if (xf86GetPciVideoInfo()) {
 	numUsed = xf86MatchPciInstances(VESA_NAME, PCI_VENDOR_GENERIC,
-					VESAChipsets, VESAPCIchipsets, 
+					VESAChipsets, VESAPCIchipsets,
 					devSections, numDevSections,
 					drv, &usedChips);
 	if (numUsed > 0) {
@@ -333,7 +383,7 @@ VESAProbe(DriverPtr drv, int flags)
 	    if ((pScrn = xf86ConfigIsaEntity(pScrn, 0,usedChips[i],
 					     VESAISAchipsets, NULL,
 					     NULL, NULL, NULL, NULL))) {
-	    
+
 		pScrn->driverVersion = VESA_VERSION;
 		pScrn->driverName    = VESA_DRIVER_NAME;
 		pScrn->name	     = VESA_NAME;
@@ -372,7 +422,7 @@ VESAFindIsaDevice(GDevPtr dev)
     outb(VGA_ATTR_DATA_W, CurrentValue ^ 0x0F);
     outb(VGA_ATTR_INDEX, 0x14 | 0x20);
     TestValue = inb(VGA_ATTR_DATA_R);
-    outb(VGA_ATTR_DATA_R, CurrentValue);
+    outb(VGA_ATTR_DATA_W, CurrentValue);
 
     /* Quit now if no VGA is present */
     if ((CurrentValue ^ 0x0F) != TestValue)
@@ -385,7 +435,7 @@ static VESAPtr
 VESAGetRec(ScrnInfoPtr pScrn)
 {
     if (!pScrn->driverPrivate)
-	pScrn->driverPrivate = xcalloc(sizeof(VESARec), 1);
+	pScrn->driverPrivate = xnfcalloc(sizeof(VESARec), 1);
 
     return ((VESAPtr)pScrn->driverPrivate);
 }
@@ -416,10 +466,11 @@ VESAFreeRec(ScrnInfoPtr pScrn)
     }
 #endif
     xfree(pVesa->monitor);
-    xfree(pVesa->vbeInfo);
+    if (pVesa->vbeInfo)
+	VBEFreeVBEInfo(pVesa->vbeInfo);
     xfree(pVesa->pal);
     xfree(pVesa->savedPal);
-    xfree(pVesa->fonts);
+    xfree(pVesa->planes);
     xfree(pScrn->driverPrivate);
     pScrn->driverPrivate = NULL;
 }
@@ -437,9 +488,10 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
     VbeModeInfoBlock *mode;
     char *mod = NULL;
     const char *reqSym = NULL;
-    Gamma gzeros = {0.0, 0.0, 0.0};
-    rgb rzeros = {0, 0, 0};
-    pointer pDDCModule;
+    static const Gamma defaultGamma = {0.0, 0.0, 0.0};
+    static const rgb defaultWeight = {0, 0, 0};
+    rgb defaultMask = {0, 0, 0};
+    ModuleDescPtr pDDCModule = NULL, pVBEModule, pMod = NULL;
     int i;
     int flags24 = 0;
     int defaultDepth = 0;
@@ -453,24 +505,16 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
     pVesa->device = xf86GetDevFromEntity(pScrn->entityList[0],
 					 pScrn->entityInstanceList[0]);
 
-#if 0
-    /* Load vgahw module */
-    if (!xf86LoadSubModule(pScrn, "vgahw"))
-    	return (FALSE);
-
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);
-#endif
-
     /* Load vbe module */
-    if (!xf86LoadSubModule(pScrn, "vbe"))
-        return (FALSE);
+    if (!(pVBEModule = xf86LoadVBEModule(pScrn)))
+	return (FALSE);
 
-    xf86LoaderReqSymLists(vbeSymbols, NULL);
+    xf86LoaderModReqSymLists(pVBEModule, vbeSymbols, NULL);
 
     if ((pVesa->pVbe = VBEExtendedInit(NULL, pVesa->pEnt->index,
-				       SET_BIOS_SCRATCH
-				       | RESTORE_BIOS_SCRATCH)) == NULL)
-        return (FALSE);
+					SET_BIOS_SCRATCH
+					| RESTORE_BIOS_SCRATCH)) == NULL)
+	return (FALSE);
 
     if (pVesa->pEnt->location.type == BUS_PCI) {
 	pVesa->pciInfo = xf86GetPciInfoForEntity(pVesa->pEnt->index);
@@ -500,17 +544,25 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 				    V_MODETYPE_VBE);
 
     /* Preferred order for default depth selection. */
+#if X_BYTE_ORDER == X_LITTLE_ENDIAN
+    /* These defaults aren't appropriate for big-endian systems */
     if (depths & V_DEPTH_16)
 	defaultDepth = 16;
-    else if (depths & V_DEPTH_15)
+    else
+    if (depths & V_DEPTH_15)
 	defaultDepth = 15;
-    else if (depths & V_DEPTH_8)
+    else
+#endif
+    if (depths & V_DEPTH_8)
 	defaultDepth = 8;
-    else if (depths & V_DEPTH_24)
+    else
+    if (depths & V_DEPTH_24)
 	defaultDepth = 24;
-    else if (depths & V_DEPTH_4)
+    else
+    if (depths & V_DEPTH_4)
 	defaultDepth = 4;
-    else if (depths & V_DEPTH_1)
+    else
+    if (depths & V_DEPTH_1)
 	defaultDepth = 1;
 
     /*
@@ -525,51 +577,129 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 	flags24 |= SupportConvert32to24 | PreferConvert32to24;
 
     if (!xf86SetDepthBpp(pScrn, defaultDepth, 0, 0, flags24)) {
-        vbeFree(pVesa->pVbe);
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
 	return (FALSE);
     }
+
     xf86PrintDepthBpp(pScrn);
 
-    /* color weight */
-    if (pScrn->depth > 8 && !xf86SetWeight(pScrn, rzeros, rzeros)) {
-        vbeFree(pVesa->pVbe);
-	return (FALSE);
+    /* options */
+    xf86CollectOptions(pScrn, NULL);
+    if (!(pVesa->Options = xnfalloc(sizeof(VESAOptions)))) {
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
+	return FALSE;
     }
-    /* visual init */
-    if (!xf86SetDefaultVisual(pScrn, -1)) {
-        vbeFree(pVesa->pVbe);
+
+    memcpy(pVesa->Options, VESAOptions, sizeof(VESAOptions));
+    xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, pVesa->Options);
+
+#if X_BYTE_ORDER == X_BIG_ENDIAN
+
+    /* The default is to assume all apertures are little-endian */
+    if (!xf86ReturnOptValBool(pVesa->Options, OPTION_VBE_BIG_ENDIAN, FALSE)) {
+	if ((pScrn->depth == 24) && (pScrn->bitsPerPixel == 24)) {
+	    /*
+	     * For bpp 24, the common layer's xf86SetWeight() sets a default of
+	     * 'RGB'.  Byte-swap this to 'BGR'.
+	     */
+	    defaultMask.red = 0x000000FF;
+	    defaultMask.green = 0x0000FF00;
+	    defaultMask.blue = 0x00FF0000;
+	} else if ((pScrn->depth == 24) && (pScrn->bitsPerPixel == 32)) {
+	    /*
+	     * For bpp 32, the common layer's xf86SetWeight() sets a default of
+	     * 'aRGB'.  Byte-swap this to 'BGRa'.
+	     */
+	    defaultMask.red = 0x0000FF00;
+	    defaultMask.green = 0x00FF0000;
+	    defaultMask.blue = 0xFF000000;
+	} else if (pScrn->bitsPerPixel > 8) {
+	    xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "Support for depth %d fbbpp"
+		       " %d requires a big-endian aperture\n",
+		       pScrn->depth, pScrn->bitsPerPixel);
+	    vbeFree(pVesa->pVbe);
+	    pVesa->pVbe = NULL;
+	    return FALSE;
+	}
+    }
+
+#endif
+
+    /* color weight */
+    if (pScrn->depth > 8 && !xf86SetWeight(pScrn, defaultWeight, defaultMask)) {
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
 	return (FALSE);
     }
 
-    xf86SetGamma(pScrn, gzeros);
+    /* visual init */
+    if (!xf86SetDefaultVisual(pScrn, -1)) {
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
+	return (FALSE);
+    }
+
+    xf86SetGamma(pScrn, defaultGamma);
 
     if (pVesa->major >= 2) {
 	/* Load ddc module */
-	if ((pDDCModule = xf86LoadSubModule(pScrn, "ddc")) == NULL) {
+	if (!(pDDCModule = xf86LoadSubModule(pScrn, "ddc"))) {
 	    vbeFree(pVesa->pVbe);
+	    pVesa->pVbe = NULL;
 	    return (FALSE);
 	}
 
+	xf86LoaderModReqSymLists(pDDCModule, ddcSymbols, NULL);
 	if ((pVesa->monitor = vbeDoEDID(pVesa->pVbe, pDDCModule)) != NULL) {
 	    xf86PrintEDID(pVesa->monitor);
 	}
 
-	xf86UnloadSubModule(pDDCModule);
     }
 
-    if ((pScrn->monitor->DDC = pVesa->monitor) != NULL)
+    if ((pScrn->monitor->DDC = pVesa->monitor) != NULL) {
 	xf86SetDDCproperties(pScrn, pVesa->monitor);
 
+	/* This test is probably not entirely accurate */
+	if (pVesa->monitor->features.input_type) {
+	    /*
+	     * For digital panels, ignore any sync tolerance specifications in
+	     * XF86Config monitor sections.  This assumes the BIOS is smart
+	     * enough to deal with the panel's tolerances by stretching and/or
+	     * centering modes, and disallowing modes larger than the panel.
+	     */
+	    if (pScrn->monitor->nHsync > 0) {
+		xf86DrvMsg(pScrn->scrnIndex, X_NOTICE,
+			   "Ignoring XF86Config HorizSync specification(s) for"
+			   " digital panel\n");
+		pScrn->monitor->nHsync = 0;
+	    }
+
+	    if (pScrn->monitor->nVrefresh > 0) {
+		xf86DrvMsg(pScrn->scrnIndex, X_NOTICE,
+			   "Ignoring XF86Config VertRefresh specification(s)"
+			   " for digital panel\n");
+		pScrn->monitor->nVrefresh = 0;
+	    }
+
+	    pScrn->monitor->flags |= MON_TOLERANCES_OPTIONAL;
+	    pVesa->defaultRefresh = TRUE;
+	}
+    }
+    if (pDDCModule)
+	xf86UnloadSubModule(pDDCModule);
+
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, DEBUG_VERB,
-			"Searching for matching VESA mode(s):\n");
+		   "Searching for matching VESA mode(s):\n");
 
     /*
      * Check the available BIOS modes, and extract those that match the
      * requirements into the modePool.  Note: modePool is a NULL-terminated
      * list.
      */
-    pScrn->modePool = VBEGetModePool (pScrn, pVesa->pVbe, pVesa->vbeInfo,
-				      V_MODETYPE_VBE);
+    pScrn->modePool = VBEGetModePool(pScrn, pVesa->pVbe, pVesa->vbeInfo,
+				     V_MODETYPE_VBE);
 
     xf86ErrorFVerb(DEBUG_VERB, "\n");
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, DEBUG_VERB,
@@ -579,13 +709,14 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
     pVesa->mapSize = vbe->TotalMemory * 65536;
     if (pScrn->modePool == NULL) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "No matching modes\n");
-        vbeFree(pVesa->pVbe);
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
 	return (FALSE);
     }
 
     VBESetModeNames(pScrn->modePool);
 
-    i = VBEValidateModes(pScrn, NULL, pScrn->display->modes, 
+    i = VBEValidateModes(pScrn, NULL, pScrn->display->modes,
 			  NULL, NULL, 0, 2048, 1, 0, 2048,
 			  pScrn->display->virtualX,
 			  pScrn->display->virtualY,
@@ -593,7 +724,8 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 
     if (i <= 0) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "No valid modes\n");
-        vbeFree(pVesa->pVbe);
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
 	return (FALSE);
     }
 
@@ -618,21 +750,13 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 
     if (pScrn->modes == NULL) {
 	xf86DrvMsg(pScrn->scrnIndex, X_ERROR, "No modes\n");
-        vbeFree(pVesa->pVbe);
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
 	return (FALSE);
     }
 
-    /* options */
-    xf86CollectOptions(pScrn, NULL);
-    if (!(pVesa->Options = xalloc(sizeof(VESAOptions)))) {
-        vbeFree(pVesa->pVbe);
-	return FALSE;
-    }
-    memcpy(pVesa->Options, VESAOptions, sizeof(VESAOptions));
-    xf86ProcessOptions(pScrn->scrnIndex, pScrn->options, pVesa->Options);
-
     /* Use shadow by default */
-    if (xf86ReturnOptValBool(pVesa->Options, OPTION_SHADOW_FB, TRUE)) 
+    if (xf86ReturnOptValBool(pVesa->Options, OPTION_SHADOW_FB, TRUE))
 	pVesa->shadowFB = TRUE;
 
     if (xf86ReturnOptValBool(pVesa->Options, OPTION_DFLT_REFRESH, FALSE))
@@ -654,9 +778,7 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 	case 0x3:	/* Planar */
 	    if (pVesa->shadowFB) {
 		mod = "fb";
-		pScrn->bitmapBitOrder = BITMAP_BIT_ORDER; 
-
-		xf86LoaderReqSymbols("fbPictureInit", NULL);
+		pScrn->bitmapBitOrder = BITMAP_BIT_ORDER;
 	    }
 	    else {
 		switch (pScrn->bitsPerPixel) {
@@ -678,7 +800,7 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 	case 0x4:	/* Packed pixel */
 	case 0x6:	/*  Direct Color */
 	    mod = "fb";
-	    pScrn->bitmapBitOrder = BITMAP_BIT_ORDER; 
+	    pScrn->bitmapBitOrder = BITMAP_BIT_ORDER;
 
 	    switch (pScrn->bitsPerPixel) {
 		case 8:
@@ -690,6 +812,7 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
 		    xf86DrvMsg(pScrn->scrnIndex, X_ERROR,
 			       "Unsupported bpp: %d", pScrn->bitsPerPixel);
 		    vbeFree(pVesa->pVbe);
+		    pVesa->pVbe = NULL;
 		    return FALSE;
 	    }
 	    break;
@@ -698,37 +821,41 @@ VESAPreInit(ScrnInfoPtr pScrn, int flags)
     if (pVesa->shadowFB) {
 	xf86DrvMsg(pScrn->scrnIndex, X_CONFIG, "Using \"Shadow Framebuffer\"\n");
 	if (pScrn->depth == 1) {
-            mod = "mfb";
+	    mod = "mfb";
 	    reqSym = "mfbScreenInit";
 	}
-	if (!xf86LoadSubModule(pScrn, "shadow")) {
+	if (!(pMod = xf86LoadSubModule(pScrn, "shadow"))) {
 	    vbeFree(pVesa->pVbe);
+	    pVesa->pVbe = NULL;
 	    return (FALSE);
 	}
-	xf86LoaderReqSymLists(shadowSymbols, NULL);
+	xf86LoaderModReqSymLists(pMod, shadowSymbols, NULL);
     }
 
-    if (mod && xf86LoadSubModule(pScrn, mod) == NULL) {
+    if (mod && !(pMod = xf86LoadSubModule(pScrn, mod))) {
 	VESAFreeRec(pScrn);
-        vbeFree(pVesa->pVbe);
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
 	return (FALSE);
     }
 
     if (mod) {
 	if (reqSym) {
-	    xf86LoaderReqSymbols(reqSym, NULL);
+	    xf86LoaderModReqSymbols(pMod, reqSym, NULL);
 	} else {
-	    xf86LoaderReqSymLists(fbSymbols, NULL);
+	    xf86LoaderModReqSymLists(pMod, fbSymbols, NULL);
 	}
     }
 
     vbeFree(pVesa->pVbe);
+    pVesa->pVbe = NULL;
 
     return (TRUE);
 }
 
 static Bool
-VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+VESAScreenInit(int scrnIndex, ScreenPtr pScreen,
+	       const int argc, const char **argv)
 {
     ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
     VESAPtr pVesa = VESAGetRec(pScrn);
@@ -736,14 +863,15 @@ VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     VbeModeInfoBlock *mode;
     int flags;
     int init_picture = 0;
+    Bool blank = FALSE;
 
     if ((pVesa->pVbe = VBEExtendedInit(NULL, pVesa->pEnt->index,
 				       SET_BIOS_SCRATCH
 				       | RESTORE_BIOS_SCRATCH)) == NULL)
-        return (FALSE);
+	return (FALSE);
 
+    mode = ((VbeModeInfoData*)(pScrn->currentMode->Private))->data;
     if (pVesa->mapPhys == 0) {
-	mode = ((VbeModeInfoData*)(pScrn->currentMode->Private))->data;
 	pScrn->videoRam = pVesa->mapSize;
 	pVesa->mapPhys = mode->PhysBasePtr;
 	pVesa->mapOff = 0;
@@ -779,8 +907,11 @@ VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
     pVesa->savedPal = VBESetGetPaletteData(pVesa->pVbe, FALSE, 0, 256,
 					    NULL, FALSE, FALSE);
 
+    if (mode->ModeAttributes & (1 << 5))
+	blank = TRUE;
+
     /* set first video mode */
-    if (!VESASetMode(pScrn, pScrn->currentMode))
+    if (!VESASetMode(pScrn, pScrn->currentMode, blank))
 	return (FALSE);
 
     /* set the viewport */
@@ -819,13 +950,13 @@ VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 	    return (FALSE);
 	case 0x3:	/* Planar */
 	    if (pVesa->shadowFB) {
-	        if (pScrn->depth == 1) {
+		if (pScrn->depth == 1) {
 		    if (!mfbScreenInit(pScreen,
 				      pVesa->shadowPtr,
 				      pScrn->virtualX, pScrn->virtualY,
 				      pScrn->xDpi, pScrn->yDpi,
 				      pScrn->displayWidth))
-		        return FALSE;
+			return FALSE;
 		} else {
 		    if (!fbScreenInit(pScreen,
 				      pVesa->shadowPtr,
@@ -973,7 +1104,7 @@ VESAScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
 
     /* Report any unused options (only for the first generation) */
     if (serverGeneration == 1)
-        xf86ShowUnusedOptions(pScrn->scrnIndex, pScrn->options);
+	xf86ShowUnusedOptions(pScrn->scrnIndex, pScrn->options);
 
     return (TRUE);
 }
@@ -983,7 +1114,7 @@ VESAEnterVT(int scrnIndex, int flags)
 {
     ScrnInfoPtr pScrn = xf86Screens[scrnIndex];
 
-    if (!VESASetMode(pScrn, pScrn->currentMode))
+    if (!VESASetMode(pScrn, pScrn->currentMode, FALSE))
 	return FALSE;
     VESAAdjustFrame(scrnIndex, pScrn->frameX0, pScrn->frameY0, 0);
     return TRUE;
@@ -1016,6 +1147,10 @@ VESACloseScreen(int scrnIndex, ScreenPtr pScreen)
 	pVesa->pDGAMode = NULL;
 	pVesa->nDGAMode = 0;
     }
+    if (pVesa->pVbe) {
+	vbeFree(pVesa->pVbe);
+	pVesa->pVbe = NULL;
+    }
     pScrn->vtSema = FALSE;
 
     pScreen->CloseScreen = pVesa->CloseScreen;
@@ -1025,12 +1160,12 @@ VESACloseScreen(int scrnIndex, ScreenPtr pScreen)
 static Bool
 VESASwitchMode(int scrnIndex, DisplayModePtr pMode, int flags)
 {
-    return VESASetMode(xf86Screens[scrnIndex], pMode);
+    return VESASetMode(xf86Screens[scrnIndex], pMode, FALSE);
 }
 
 /* Set a graphics mode */
 static Bool
-VESASetMode(ScrnInfoPtr pScrn, DisplayModePtr pMode)
+VESASetMode(ScrnInfoPtr pScrn, DisplayModePtr pMode, Bool blank)
 {
     VESAPtr pVesa;
     VbeModeInfoData *data;
@@ -1040,7 +1175,9 @@ VESASetMode(ScrnInfoPtr pScrn, DisplayModePtr pMode)
 
     data = (VbeModeInfoData*)pMode->Private;
 
-    mode = data->mode | (1 << 15);
+    mode = data->mode;
+    if (!blank)
+	mode |= (1 << 15);
 
     /* enable linear addressing */
     if (pVesa->mapPhys != 0xa0000)
@@ -1071,7 +1208,7 @@ VESASetMode(ScrnInfoPtr pScrn, DisplayModePtr pMode)
     if (data->data->XResolution != pScrn->displayWidth)
 	VBESetLogicalScanline(pVesa->pVbe, pScrn->displayWidth);
 
-    if (pScrn->bitsPerPixel >= 8 && pVesa->vbeInfo->Capabilities[0] & 0x01)
+    if (pScrn->bitsPerPixel == 8 && pVesa->vbeInfo->Capabilities[0] & 0x01)
 	VBESetGetDACPaletteFormat(pVesa->pVbe, 8);
 
     pScrn->vtSema = TRUE;
@@ -1104,31 +1241,33 @@ VESAMapVidMem(ScrnInfoPtr pScrn)
     pScrn->memPhysBase = pVesa->mapPhys;
     pScrn->fbOffset = pVesa->mapOff;
 
-    if (pVesa->mapPhys != 0xa0000 && pVesa->pEnt->location.type == BUS_PCI)
-	pVesa->base = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
-				    pVesa->pciTag, pScrn->memPhysBase,
-				    pVesa->mapSize);
-    else
-	pVesa->base = xf86MapDomainMemory(pScrn->scrnIndex, 0, pVesa->pciTag,
-					  pScrn->memPhysBase, pVesa->mapSize);
-
-    if (pVesa->base) {
-	if (pVesa->mapPhys != 0xa0000)
-	    pVesa->VGAbase = xf86MapDomainMemory(pScrn->scrnIndex, 0,
-						 pVesa->pciTag,
-						 0xa0000, 0x10000);
-	else
-	    pVesa->VGAbase = pVesa->base;
-    }
-
     pVesa->ioBase = pScrn->domainIOBase;
 
+    if (pVesa->pEnt->location.type == BUS_PCI)
+	pVesa->VGAbase = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_MMIO,
+				       pVesa->pciTag, 0x0a0000, 0x010000);
+    else
+	pVesa->VGAbase = xf86MapVidMem(pScrn->scrnIndex, VIDMEM_MMIO,
+				       0x0a0000, 0x010000);
+
+    if (pVesa->VGAbase != NULL) {
+	if (pVesa->mapPhys == 0x0a0000)
+	    pVesa->base = pVesa->VGAbase;
+	else if (pVesa->pEnt->location.type == BUS_PCI)
+	    pVesa->base = xf86MapPciMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
+					pVesa->pciTag, pScrn->memPhysBase,
+					pVesa->mapSize);
+	else
+	    pVesa->base = xf86MapVidMem(pScrn->scrnIndex, VIDMEM_FRAMEBUFFER,
+					pScrn->memPhysBase, pVesa->mapSize);
+    }
+
     xf86DrvMsgVerb(pScrn->scrnIndex, X_INFO, DEBUG_VERB,
 		   "virtual address = %p,\n"
 		   "\tphysical address = 0x%lx, size = %ld\n",
 		   pVesa->base, pScrn->memPhysBase, pVesa->mapSize);
 
-    return (pVesa->base != NULL);
+    return pVesa->base != NULL;
 }
 
 static void
@@ -1210,7 +1349,7 @@ VESALoadPalette(ScrnInfoPtr pScrn, int numColors, int *indices,
     int base;
 
     if (pVesa->pal == NULL)
-	pVesa->pal = xcalloc(1, sizeof(CARD32) * 256);
+	pVesa->pal = xnfcalloc(1, sizeof(CARD32) * 256);
 
     for (i = 0, base = idx = indices[i]; i < numColors; i++, idx++) {
 	int j = indices[i];
@@ -1325,12 +1464,12 @@ SeqReset(VESAPtr pVesa, Bool start)
 }
 
 static void
-SaveFonts(ScrnInfoPtr pScrn)
+VESASavePlanes(ScrnInfoPtr pScrn)
 {
     VESAPtr pVesa = VESAGetRec(pScrn);
-    unsigned char miscOut, attr10, gr4, gr5, gr6, seq2, seq4, scrn;
+    CARD8 miscOut, attr10, gr1, gr3, gr4, gr5, gr6, gr8, seq2, seq4, scrn;
 
-    if (pVesa->fonts != NULL)
+    if (pVesa->planes != NULL)
 	return;
 
     /* If in graphics mode, don't save anything */
@@ -1338,13 +1477,22 @@ SaveFonts(ScrnInfoPtr pScrn)
     if (attr10 & 0x01)
 	return;
 
-    pVesa->fonts = xalloc(16384);
+    /* Ensure bank 0 (ignoring error returns) */
+    (void) VBEBankSwitch(pVesa->pVbe, 0, 0);
+    (void) VBEBankSwitch(pVesa->pVbe, 0, 1);
+    pVesa->curBank = -1;
+
+    /* Save the first 64K of all four planes */
+    pVesa->planes = xnfalloc(4 * 0x010000);
 
     /* save the registers that are needed here */
     miscOut = ReadMiscOut();
+    gr1 = ReadGr(pVesa, 0x01);
+    gr3 = ReadGr(pVesa, 0x03);
     gr4 = ReadGr(pVesa, 0x04);
     gr5 = ReadGr(pVesa, 0x05);
     gr6 = ReadGr(pVesa, 0x06);
+    gr8 = ReadGr(pVesa, 0x08);
     seq2 = ReadSeq(pVesa, 0x02);
     seq4 = ReadSeq(pVesa, 0x04);
 
@@ -1357,22 +1505,32 @@ SaveFonts(ScrnInfoPtr pScrn)
     SeqReset(pVesa, FALSE);
 
     WriteAttr(pVesa, 0x10, 0x01);	/* graphics mode */
+    WriteSeq(0x04, 0x06);		/* enable plane graphics */
+    WriteGr(0x01, 0x00);		/* all planes come from CPU */
+    WriteGr(0x03, 0x00);		/* don't rotate, write unmodified */
+    WriteGr(0x05, 0x00);		/* write mode 0, read mode 0 */
+    WriteGr(0x06, 0x05);		/* set graphics */
+    WriteGr(0x08, 0xFF);		/* write all bits in a byte */
 
-    /*font1 */
-    WriteSeq(0x02, 0x04);	/* write to plane 2 */
-    WriteSeq(0x04, 0x06);	/* enable plane graphics */
-    WriteGr(0x04, 0x02);	/* read plane 2 */
-    WriteGr(0x05, 0x00);	/* write mode 0, read mode 0 */
-    WriteGr(0x06, 0x05);	/* set graphics */
-    slowbcopy_frombus(pVesa->VGAbase, pVesa->fonts, 8192);
+    /* plane0 */
+    WriteSeq(0x02, 0x01);		/* write to plane 0 */
+    WriteGr(0x04, 0x00);		/* read plane 0 */
+    slowbcopy_frombus(pVesa->VGAbase, pVesa->planes + 0 * 0x010000, 0x010000);
 
-    /* font2 */
-    WriteSeq(0x02, 0x08);	/* write to plane 3 */
-    WriteSeq(0x04, 0x06);	/* enable plane graphics */
-    WriteGr(0x04, 0x03);	/* read plane 3 */
-    WriteGr(0x05, 0x00);	/* write mode 0, read mode 0 */
-    WriteGr(0x06, 0x05);	/* set graphics */
-    slowbcopy_frombus(pVesa->VGAbase, pVesa->fonts + 8192, 8192);
+    /* plane1 */
+    WriteSeq(0x02, 0x02);		/* write to plane 1 */
+    WriteGr(0x04, 0x01);		/* read plane 1 */
+    slowbcopy_frombus(pVesa->VGAbase, pVesa->planes + 1 * 0x010000, 0x010000);
+
+    /* plane2 */
+    WriteSeq(0x02, 0x04);		/* write to plane 2 */
+    WriteGr(0x04, 0x02);		/* read plane 2 */
+    slowbcopy_frombus(pVesa->VGAbase, pVesa->planes + 2 * 0x010000, 0x010000);
+
+    /* plane3 */
+    WriteSeq(0x02, 0x08);		/* write to plane 3 */
+    WriteGr(0x04, 0x03);		/* read plane 3 */
+    slowbcopy_frombus(pVesa->VGAbase, pVesa->planes + 3 * 0x010000, 0x010000);
 
     scrn = ReadSeq(pVesa, 0x01) & ~0x20;
     SeqReset(pVesa, TRUE);
@@ -1383,23 +1541,28 @@ SaveFonts(ScrnInfoPtr pScrn)
     WriteAttr(pVesa, 0x10, attr10);
     WriteSeq(0x02, seq2);
     WriteSeq(0x04, seq4);
+    WriteGr(0x01, gr1);
+    WriteGr(0x03, gr3);
     WriteGr(0x04, gr4);
     WriteGr(0x05, gr5);
     WriteGr(0x06, gr6);
+    WriteGr(0x08, gr8);
     WriteMiscOut(miscOut);
 }
 
 static void
-RestoreFonts(ScrnInfoPtr pScrn)
+VESARestorePlanes(ScrnInfoPtr pScrn)
 {
     VESAPtr pVesa = VESAGetRec(pScrn);
-    unsigned char miscOut, attr10, gr1, gr3, gr4, gr5, gr6, gr8, seq2, seq4, scrn;
+    CARD8 miscOut, attr10, gr1, gr3, gr4, gr5, gr6, gr8, seq2, seq4, scrn;
 
-    if (pVesa->fonts == NULL)
+    if (pVesa->planes == NULL)
 	return;
 
-    if (pVesa->mapPhys == 0xa0000 && pVesa->curBank != 0)
-	VESABankSwitch(pScrn->pScreen, 0);
+    /* Ensure bank 0 (ignoring error returns) */
+    (void) VBEBankSwitch(pVesa->pVbe, 0, 0);
+    (void) VBEBankSwitch(pVesa->pVbe, 0, 1);
+    pVesa->curBank = -1;
 
     /* save the registers that are needed here */
     miscOut = ReadMiscOut();
@@ -1422,26 +1585,28 @@ RestoreFonts(ScrnInfoPtr pScrn)
     SeqReset(pVesa, FALSE);
 
     WriteAttr(pVesa, 0x10, 0x01);	/* graphics mode */
-    if (pScrn->depth == 4) {
-	/* GJA */
-	WriteGr(0x03, 0x00);	/* don't rotate, write unmodified */
-	WriteGr(0x08, 0xFF);	/* write all bits in a byte */
-	WriteGr(0x01, 0x00);	/* all planes come from CPU */
-    }
+    WriteSeq(0x04, 0x06);		/* enable plane graphics */
+    WriteGr(0x01, 0x00);		/* all planes come from CPU */
+    WriteGr(0x03, 0x00);		/* don't rotate, write unmodified */
+    WriteGr(0x05, 0x00);		/* write mode 0, read mode 0 */
+    WriteGr(0x06, 0x05);		/* set graphics */
+    WriteGr(0x08, 0xFF);		/* write all bits in a byte */
 
-    WriteSeq(0x02, 0x04);   /* write to plane 2 */
-    WriteSeq(0x04, 0x06);   /* enable plane graphics */
-    WriteGr(0x04, 0x02);    /* read plane 2 */
-    WriteGr(0x05, 0x00);    /* write mode 0, read mode 0 */
-    WriteGr(0x06, 0x05);    /* set graphics */
-    slowbcopy_tobus(pVesa->fonts, pVesa->VGAbase, 8192);
+    WriteSeq(0x02, 0x01);		/* write to plane 0 */
+    WriteGr(0x04, 0x00);		/* read plane 0 */
+    slowbcopy_tobus(pVesa->planes + 0 * 0x010000, pVesa->VGAbase, 0x010000);
 
-    WriteSeq(0x02, 0x08);   /* write to plane 3 */
-    WriteSeq(0x04, 0x06);   /* enable plane graphics */
-    WriteGr(0x04, 0x03);    /* read plane 3 */
-    WriteGr(0x05, 0x00);    /* write mode 0, read mode 0 */
-    WriteGr(0x06, 0x05);    /* set graphics */
-    slowbcopy_tobus(pVesa->fonts + 8192, pVesa->VGAbase, 8192);
+    WriteSeq(0x02, 0x02);		/* write to plane 1 */
+    WriteGr(0x04, 0x01);		/* read plane 1 */
+    slowbcopy_tobus(pVesa->planes + 1 * 0x010000, pVesa->VGAbase, 0x010000);
+
+    WriteSeq(0x02, 0x04);		/* write to plane 2 */
+    WriteGr(0x04, 0x02);		/* read plane 2 */
+    slowbcopy_tobus(pVesa->planes + 2 * 0x010000, pVesa->VGAbase, 0x010000);
+
+    WriteSeq(0x02, 0x08);		/* write to plane 3 */
+    WriteGr(0x04, 0x03);		/* read plane 3 */
+    slowbcopy_tobus(pVesa->planes + 3 * 0x010000, pVesa->VGAbase, 0x010000);
 
     scrn = ReadSeq(pVesa, 0x01) & ~0x20;
     SeqReset(pVesa, TRUE);
@@ -1467,6 +1632,18 @@ VESASaveScreen(ScreenPtr pScreen, int mode)
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
     VESAPtr pVesa = VESAGetRec(pScrn);
     Bool on = xf86IsUnblank(mode);
+    VbeModeInfoBlock *vMode;
+
+    if (pScrn->currentMode) {
+	vMode = ((VbeModeInfoData*)pScrn->currentMode->Private)->data;
+	/*
+	 * Don't try to use VGA blanking for modes that are not
+	 * VGA-compatible.
+	 */
+	if (vMode->ModeAttributes & (1 << 5)) {
+	    return FALSE;
+	}
+    }
 
     if (on)
 	SetTimeSinceLastInputEvent();
@@ -1486,7 +1663,7 @@ VESASaveScreen(ScreenPtr pScreen, int mode)
     return (TRUE);
 }
 
-static int 
+static int
 VESABankSwitch(ScreenPtr pScreen, unsigned int iBank)
 {
     ScrnInfoPtr pScrn = xf86Screens[pScreen->myNum];
@@ -1495,9 +1672,9 @@ VESABankSwitch(ScreenPtr pScreen, unsigned int iBank)
     if (pVesa->curBank == iBank)
 	return (0);
     if (!VBEBankSwitch(pVesa->pVbe, iBank, 0))
-        return (1);
+	return (1);
     if (pVesa->bankSwitchWindowB) {
-        if (!VBEBankSwitch(pVesa->pVbe, iBank, 1))
+	if (!VBEBankSwitch(pVesa->pVbe, iBank, 1))
 	   return (1);
     }
     pVesa->curBank = iBank;
@@ -1510,7 +1687,7 @@ VESASaveRestore(ScrnInfoPtr pScrn, vbeSaveRestoreFunction function)
 {
     VESAPtr pVesa;
 
-    if (MODE_QUERY < 0 || function > MODE_RESTORE)
+    if ((function < MODE_QUERY) || (function > MODE_RESTORE))
 	return (FALSE);
 
     pVesa = VESAGetRec(pScrn);
@@ -1522,21 +1699,26 @@ VESASaveRestore(ScrnInfoPtr pScrn, vbeSaveRestoreFunction function)
 
 	/* Make sure we save at least this information in case of failure */
 	(void)VBEGetVBEMode(pVesa->pVbe, &pVesa->stateMode);
-	SaveFonts(pScrn);
+	VESASavePlanes(pScrn);
 
-	if (pVesa->major > 1) {
+	/* This only for VESA modes */
+	if ((pVesa->stateMode & 0x0100) && (pVesa->major > 1)) {
 	    if (!VBESaveRestore(pVesa->pVbe,function,(pointer)&pVesa->state,
 				&pVesa->stateSize,&pVesa->statePage))
-	        return FALSE;
+		return FALSE;
 
 	}
     }
 
     /* Save/Restore Super VGA state */
     if (function != MODE_QUERY) {
-        Bool retval = TRUE;
+	Bool retval = TRUE;
 
-	if (pVesa->major > 1) {
+	if (function == MODE_RESTORE)
+	    VBESetVBEMode(pVesa->pVbe, pVesa->stateMode, NULL);
+
+	/* This only for VESA modes */
+	if ((pVesa->stateMode & 0x0100) && (pVesa->major > 1)) {
 	    if (function == MODE_RESTORE)
 		memcpy(pVesa->state, pVesa->pstate, pVesa->stateSize);
 
@@ -1544,17 +1726,15 @@ VESASaveRestore(ScrnInfoPtr pScrn, vbeSaveRestoreFunction function)
 					 (pointer)&pVesa->state,
 					 &pVesa->stateSize,&pVesa->statePage))
 		&& function == MODE_SAVE) {
-	        /* don't rely on the memory not being touched */
-	        if (pVesa->pstate == NULL)
-		    pVesa->pstate = xalloc(pVesa->stateSize);
+		/* don't rely on the memory not being touched */
+		if (pVesa->pstate == NULL)
+		    pVesa->pstate = xnfalloc(pVesa->stateSize);
 		memcpy(pVesa->pstate, pVesa->state, pVesa->stateSize);
 	    }
 	}
 
-	if (function == MODE_RESTORE) {
-	    VBESetVBEMode(pVesa->pVbe, pVesa->stateMode, NULL);
-	    RestoreFonts(pScrn);
-	}
+	if (function == MODE_RESTORE)
+	    VESARestorePlanes(pScrn);
 
 	if (!retval)
 	    return (FALSE);
@@ -1566,7 +1746,7 @@ VESASaveRestore(ScrnInfoPtr pScrn, vbeSaveRestoreFunction function)
 
 static void
 VESADisplayPowerManagementSet(ScrnInfoPtr pScrn, int mode,
-                int flags)
+		int flags)
 {
     VESAPtr pVesa = VESAGetRec(pScrn);
     unsigned char seq1 = 0, crtc17 = 0;
@@ -1612,24 +1792,25 @@ VESADisplayPowerManagementSet(ScrnInfoPtr pScrn, int mode,
  * DGA stuff
  ***********************************************************************/
 static Bool VESADGAOpenFramebuffer(ScrnInfoPtr pScrn, char **DeviceName,
-				   unsigned char **ApertureBase,
-				   int *ApertureSize, int *ApertureOffset,
-				   int *flags);
+				   unsigned int *ApertureBase,
+				   unsigned int *ApertureSize,
+				   unsigned int *ApertureOffset,
+				   unsigned int *flags);
 static Bool VESADGASetMode(ScrnInfoPtr pScrn, DGAModePtr pDGAMode);
 static void VESADGASetViewport(ScrnInfoPtr pScrn, int x, int y, int flags);
 
 static Bool
 VESADGAOpenFramebuffer(ScrnInfoPtr pScrn, char **DeviceName,
-		       unsigned char **ApertureBase, int *ApertureSize,
-		       int *ApertureOffset, int *flags)
+		       unsigned int *ApertureBase, unsigned int *ApertureSize,
+		       unsigned int *ApertureOffset, unsigned int *flags)
 {
     VESAPtr pVesa = VESAGetRec(pScrn);
 
     *DeviceName = NULL;		/* No special device */
-    *ApertureBase = (unsigned char *)(long)(pVesa->mapPhys);
+    *ApertureBase = pVesa->mapPhys;
     *ApertureSize = pVesa->mapSize;
     *ApertureOffset = pVesa->mapOff;
-    *flags = DGA_NEED_ROOT;
+    *flags = 0;
 
     return (TRUE);
 }
