@@ -2,11 +2,7 @@
  * Copyright (C) 1998-2001 VMware, Inc.
  * All Rights Reserved
  * **********************************************************/
-#ifdef VMX86_DEVEL
-char rcsId_vmware[] =
-    "Id: vmware.c,v 1.11 2001/02/23 02:10:39 yoel Exp $";
-#endif
-/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/vmware/vmware.c,v 1.24 2005/02/26 01:07:13 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/drivers/vmware/vmware.c,v 1.29 2007/01/23 18:03:11 tsi Exp $ */
 
 /*
  * TODO: support the vmware linux kernel fb driver (Option "UseFBDev").
@@ -371,6 +367,10 @@ RewriteTagString(const char *istr, char *ostr, int osize)
 static void
 VMWAREIdentify(int flags)
 {
+    char buildString[sizeof(VMWAREBuildStr)];
+
+    RewriteTagString(VMWAREBuildStr, buildString, sizeof(VMWAREBuildStr));
+    xf86MsgVerb(X_PROBED, 4, "%s", buildString);
     xf86PrintChipsets(VMWARE_NAME, "driver for VMware SVGA", VMWAREChipsets);
 }
 
@@ -391,6 +391,7 @@ VMWAREPreInit(ScrnInfoPtr pScrn, int flags)
     int i;
     ClockRange* clockRanges;
     IOADDRESS domainIOBase = 0;
+    ModuleDescPtr pMod;
 
 #ifndef BUILD_FOR_420
     domainIOBase = pScrn->domainIOBase;
@@ -436,11 +437,11 @@ VMWAREPreInit(ScrnInfoPtr pScrn, int flags)
                "VMware SVGA regs at (0x%04lx, 0x%04lx)\n",
                pVMWARE->indexReg, pVMWARE->valueReg);
 
-    if (!xf86LoadSubModule(pScrn, "vgahw")) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "vgahw"))) {
         return FALSE;
     }
 
-    xf86LoaderReqSymLists(vgahwSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, vgahwSymbols, NULL);
 
     if (!vgaHWGetHWRec(pScrn)) {
         return FALSE;
@@ -761,28 +762,32 @@ VMWAREPreInit(ScrnInfoPtr pScrn, int flags)
     pScrn->currentMode = pScrn->modes;
     xf86PrintModes(pScrn);
     xf86SetDpi(pScrn, 0, 0);
-    if (!xf86LoadSubModule(pScrn, "fb") ||
-        !xf86LoadSubModule(pScrn, "shadowfb")) {
+    if (!(pMod = xf86LoadSubModule(pScrn, "fb"))) {
         VMWAREFreeRec(pScrn);
         return FALSE;
     }
-    xf86LoaderReqSymLists(fbSymbols, shadowfbSymbols, NULL);
+    xf86LoaderModReqSymLists(pMod, fbSymbols, NULL);
+    if (!(pMod = xf86LoadSubModule(pScrn, "shadowfb"))) {
+        VMWAREFreeRec(pScrn);
+        return FALSE;
+    }
+    xf86LoaderModReqSymLists(pMod, shadowfbSymbols, NULL);
 
     /* Need ramdac for hwcursor */
     if (pVMWARE->hwCursor) {
-        if (!xf86LoadSubModule(pScrn, "ramdac")) {
+        if (!(pMod = xf86LoadSubModule(pScrn, "ramdac"))) {
             VMWAREFreeRec(pScrn);
             return FALSE;
         }
-        xf86LoaderReqSymLists(ramdacSymbols, NULL);
+        xf86LoaderModReqSymLists(pMod, ramdacSymbols, NULL);
     }
 
     if (!pVMWARE->noAccel) {
-        if (!xf86LoadSubModule(pScrn, "xaa")) {
+        if (!(pMod = xf86LoadSubModule(pScrn, "xaa"))) {
             VMWAREFreeRec(pScrn);
             return FALSE;
         }
-        xf86LoaderReqSymLists(vmwareXaaSymbols, NULL);
+        xf86LoaderModReqSymLists(pMod, vmwareXaaSymbols, NULL);
     }
 
     return TRUE;
@@ -1124,7 +1129,8 @@ VMWARELoadPalette(ScrnInfoPtr pScrn, int numColors, int* indices,
 }
 
 static Bool
-VMWAREScreenInit(int scrnIndex, ScreenPtr pScreen, int argc, char **argv)
+VMWAREScreenInit(int scrnIndex, ScreenPtr pScreen,
+                 const int argc, const char **argv)
 {
     ScrnInfoPtr pScrn;
     vgaHWPtr hwp;
@@ -1360,10 +1366,6 @@ VMWAREProbe(DriverPtr drv, int flags)
     int *usedChips;
     int i;
     Bool foundScreen = FALSE;
-    char buildString[sizeof(VMWAREBuildStr)];
-
-    RewriteTagString(VMWAREBuildStr, buildString, sizeof(VMWAREBuildStr));
-    xf86MsgVerb(X_PROBED, 4, "%s", buildString);
 
     numDevSections = xf86MatchDevice(VMWARE_DRIVER_NAME, &devSections);
     if (numDevSections <= 0) {
@@ -1428,7 +1430,7 @@ static MODULESETUPPROTO(vmwareSetup);
 XF86ModuleData vmwareModuleData = { &vmwareVersRec, vmwareSetup, NULL };
 
 static pointer
-vmwareSetup(pointer module, pointer opts, int *errmaj, int *errmin)
+vmwareSetup(ModuleDescPtr module, pointer opts, int *errmaj, int *errmin)
 {
     static Bool setupDone = FALSE;
 
@@ -1436,8 +1438,8 @@ vmwareSetup(pointer module, pointer opts, int *errmaj, int *errmin)
         setupDone = TRUE;
         xf86AddDriver(&VMWARE, module, 0);
 
-        LoaderRefSymLists(vgahwSymbols, fbSymbols, ramdacSymbols,
-                          shadowfbSymbols, vmwareXaaSymbols, NULL);
+        LoaderModRefSymLists(module, vgahwSymbols, fbSymbols, ramdacSymbols,
+                             shadowfbSymbols, vmwareXaaSymbols, NULL);
 
         return (pointer)1;
     }
