@@ -21,12 +21,13 @@
  * PERFORMANCE OF THIS SOFTWARE.
  */
 
-/* $XFree86: xc/programs/Xserver/hw/xfree86/loader/hash.h,v 1.3 1998/07/25 16:56:16 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/hw/xfree86/loader/hash.h,v 1.4 2006/03/02 03:00:38 dawes Exp $ */
 
 #ifndef _HASH_H
 #define _HASH_H
 
-#include "loader.h"
+#define HASHDIV 12
+#define HASHSIZE (1 << HASHDIV)
 
 typedef struct _HashIterator {
     itemPtr pItem;
