@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/include/os.h,v 3.68 2006/03/16 21:44:00 dawes Exp $ */
+/* $XFree86: xc/programs/Xserver/include/os.h,v 3.71 2007/04/03 00:21:13 tsi Exp $ */
 /***********************************************************
 
 Copyright 1987, 1998  The Open Group
@@ -292,7 +292,7 @@ extern SIGVAL AbortServer(int /*sig*/) NORET;
 
 extern void UseMsg(void);
 
-extern void ProcessCommandLine(int /*argc*/, char* /*argv*/[]);
+extern void ProcessCommandLine(const int /*argc*/, const char* /*argv*/[]);
 
 extern int set_font_authorizations(
     char ** /* authorizations */, 
@@ -358,10 +358,10 @@ void OsBlockSignals (void);
 void OsReleaseSignals (void);
 
 #if !defined(WIN32) && !defined(__UNIXOS2__)
-extern int System(char *);
-extern pointer Popen(char *, char *);
+extern int System(const char *);
+extern pointer Popen(const char *, const char *);
 extern int Pclose(pointer);
-extern pointer Fopen(char *, char *);
+extern pointer Fopen(const char *, const char *);
 extern int Fclose(pointer);
 #else
 #define System(a) system(a)
@@ -371,7 +371,7 @@ extern int Fclose(pointer);
 #define Fclose(a) fclose(a)
 #endif
 
-extern void CheckUserParameters(int argc, char **argv, char **envp);
+extern void CheckUserParameters(const int argc, const char **argv, char **envp);
 extern void CheckUserAuthorization(void);
 
 extern int AddHost(
@@ -415,7 +415,7 @@ extern int GetAccessControl(void);
 
 extern void AddLocalHosts(void);
 
-extern void ResetHosts(char *display);
+extern void ResetHosts(const char *display);
 
 extern void EnableLocalHost(void);
 
@@ -427,7 +427,7 @@ extern void DefineSelf(int /*fd*/);
 
 extern void AugmentSelf(pointer /*from*/, int /*len*/);
 
-extern void InitAuthorization(char * /*filename*/);
+extern void InitAuthorization(const char * /*filename*/);
 
 /* extern int LoadAuthorization(void); */
 
@@ -481,7 +481,7 @@ extern XID GenerateAuthorization(
 extern void ExpandCommandLine(int * /*pargc*/, char *** /*pargv*/);
 #endif
 
-extern int ddxProcessArgument(int /*argc*/, char * /*argv*/ [], int /*i*/);
+extern int ddxProcessArgument(int /*argc*/, const char * /*argv*/ [], int /*i*/);
 
 extern void ddxUseMsg(void);
 
@@ -595,6 +595,12 @@ __attribute((noreturn))
 #endif
 ;
 
+#ifdef __GNUC__
+#define FatalAlloc()	FatalError("Out of Memory: %s() %s:%d", __FUNCTION__, __FILE__, __LINE__); 
+#else
+#define FatalAlloc()	FatalError("Out of Memory: %s:%d", __FILE__, __LINE__); 
+#endif
+
 extern void VErrorF(const char *f, va_list args);
 extern void ErrorF(const char *f, ...) _printf_attribute(1,2);
 extern void Error(char *str);
@@ -621,7 +627,13 @@ extern int Xasprintf(char **ret, const char *format, ...)
 #if defined(printf_is_xf86printf) && !defined(printf)
 #define printf xf86printf
 #endif
+
 extern int getArgc(void);
+extern const char **getArgvp(void);
 extern const char *getArgv(int i);
 
+#ifdef __DARWIN__
+extern void DarwinHandleGUI(int argc, const char *argv[], char *envp[]);
+#endif
+
 #endif /* OS_H */
