@@ -1,4 +1,4 @@
-/* $XFree86: xc/programs/Xserver/lbx/lbxmain.c,v 1.14 2005/10/14 15:17:20 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/lbx/lbxmain.c,v 1.15 2007/01/04 02:48:13 tsi Exp $ */
 /*
 
 Copyright 1996, 1998  The Open Group
@@ -118,7 +118,7 @@ int lbxDebug = 0;
 
 
 void
-LbxExtensionInit(void)
+LbxExtensionInit(INITARGS)
 {
     ExtensionEntry *extEntry;
 
