@@ -44,7 +44,7 @@ ARISING OUT OF OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS
 SOFTWARE.
 
 ******************************************************************/
-/* $XFree86: xc/programs/Xserver/os/access.c,v 3.60 2005/10/14 15:17:26 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/os/access.c,v 3.63 2007/04/09 15:37:19 tsi Exp $ */
 
 #ifdef WIN32
 #include <X11/Xwinsock.h>
@@ -518,7 +518,8 @@ DefineSelf (int fd)
      * see), whereas gethostname() kindly truncates it for me.
      */
 #ifndef QNX4
-    uname(&name);
+    if (uname(&name) < 0)
+	name.nodename[0] = '\0';
 #else
     /* QNX4's uname returns node number in name.nodename, not the hostname
        have to overwrite it */
@@ -1062,7 +1063,7 @@ AddLocalHosts (void)
 
 /* Reset access control list to initial hosts */
 void
-ResetHosts (char *display)
+ResetHosts (const char *display)
 {
     HOST	*host;
     char                lhostname[120], ohostname[120];
@@ -1683,6 +1684,7 @@ ConvertAddr (
     int				*len,
     pointer			*addr)
 {
+    *addr = NULL;
     if (*len == 0)
         return (FamilyLocal);
     switch (saddr->sa_family)
