@@ -1,5 +1,5 @@
 /*
- * $XFree86: xc/programs/Xserver/randr/randr.c,v 1.23 2005/10/14 15:17:27 tsi Exp $
+ * $XFree86: xc/programs/Xserver/randr/randr.c,v 1.25 2007/04/09 15:37:19 tsi Exp $
  *
  * Copyright  2000, Compaq Computer Corporation, 
  * Copyright  2002, Hewlett Packard, Inc.
@@ -276,7 +276,7 @@ RRFreeEvents (pointer data, XID id)
 }
 
 void
-RRExtensionInit (void)
+RRExtensionInit (INITARGS)
 {
     ExtensionEntry *extEntry;
 
@@ -1147,6 +1147,7 @@ RRRegisterSize (ScreenPtr	    pScreen,
     tmp.nRatesInUse = 0;
     tmp.referenced = TRUE;
     tmp.oldReferenced = FALSE;
+    tmp.id = 0;
     for (i = 0; i < pScrPriv->nSizes; i++)
 	if (RRScreenSizeMatches (&tmp, &pScrPriv->pSizes[i]))
 	{
