@@ -23,7 +23,7 @@ OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION  WITH
 THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 ********************************************************/
-/* $XFree86: xc/programs/Xserver/xkb/xkbEvents.c,v 3.14 2005/10/14 15:17:28 tsi Exp $ */
+/* $XFree86: xc/programs/Xserver/xkb/xkbEvents.c,v 3.15 2006/12/10 15:58:33 tsi Exp $ */
 
 #include <stdio.h>
 #define NEED_EVENTS 1
@@ -809,8 +809,9 @@ XkbSrvInfoPtr	xkbi;
 		((xE[0].u.u.type==KeyPress)||(xE[0].u.u.type==KeyRelease))) {
 	    ErrorF("XKbFilterWriteEvents:\n");
 	    ErrorF("   Event state= 0x%04x\n",xE[0].u.keyButtonPointer.state);
-	    ErrorF("   XkbLastRepeatEvent!=xE (0x%x!=0x%x) %s\n",
-			XkbLastRepeatEvent,xE,
+	    ErrorF("   XkbLastRepeatEvent!=xE (0x%lx!=0x%lx) %s\n",
+			(unsigned long)XkbLastRepeatEvent,
+			(unsigned long)xE,
 			((XkbLastRepeatEvent!=(pointer)xE)?"True":"False"));
 	    ErrorF("   (xkbClientEventsFlags&XWDA)==0 (0x%x) %s\n",
 		pClient->xkbClientFlags,
